@@ -10,7 +10,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 
@@ -23,7 +25,6 @@
 /*----------------------------------------------------------------------*/
 
 #include "xcircuit.h"
-#include "cursors.h"
 #include "colordefs.h"
 
 /*----------------------------------------------------------------------*/
@@ -264,7 +265,7 @@ labelptr findlabelcopy(labelptr curlabel, stringpart *curstring)
 void labeltext(int keypressed, char *clientdata)
 {
    labelptr curlabel;
-   stringpart *curpos, *strptr;
+   stringpart *curpos;
    int locpos;
    Boolean do_redraw = False;
    short tmplength, tmpheight, cfont;
@@ -282,7 +283,6 @@ void labeltext(int keypressed, char *clientdata)
    UDrawTLine(curlabel);
 
    if (isbound(keypressed, XCF_Text_Delete) && textpos > 1) {
-      short k;
       int curloc, curpos;
       stringpart *strptr;
 
@@ -641,18 +641,14 @@ void splinebutton(XButtonEvent *event)
 
 void trackspline(Widget w, caddr_t clientdata, caddr_t calldata)
 {
-   Window 	nullwin;
-   int    	nullint, xpos, ypos;
    XPoint       newpos;
-   u_int		nullui;
    splineptr    newspline;
 
    newspline = (eventmode == EPATH_MODE) ?
                (splineptr)(*((*((pathptr *)EDITPART))->plist
 		+ areastruct.editsubpart)) : TOSPLINE(EDITPART);
-   XQueryPointer(dpy, win, &nullwin, &nullwin, &nullint, &nullint, &xpos, &ypos,
-	&nullui);
-   snap(xpos, ypos, &newpos);
+   newpos = UGetCursorPos();
+   u2u_snap(&newpos);
    if (areastruct.save.x == newpos.x && areastruct.save.y == newpos.y) return;
 
    UDrawEditSpline(areastruct.topobject, newspline);
@@ -735,20 +731,16 @@ void arcbutton(XButtonEvent *event)
 
 void trackarc(Widget w, caddr_t clientdata, caddr_t calldata)
 {
-   Window nullwin;
-   int    nullint, xpos, ypos;
    XPoint newpos;
-   u_int   nullui;
    arcptr newarc;
    double adjrat;
 
    newarc = (eventmode == EPATH_MODE) ?
 	    (arcptr)(*((*((pathptr *)EDITPART))->plist
 	    + areastruct.editsubpart)) : TOARC(EDITPART);
-   XQueryPointer(dpy, win, &nullwin, &nullwin, &nullint, &nullint, &xpos, &ypos,
-	&nullui);
 
-   snap(xpos, ypos, &newpos);
+   newpos = UGetCursorPos();
+   u2u_snap(&newpos);
    if (areastruct.save.x == newpos.x && areastruct.save.y == newpos.y) return;
 
    UDrawArc(areastruct.topobject, newarc);
@@ -861,18 +853,14 @@ void boxbutton(XButtonEvent *event)
 
 void trackbox(Widget w, caddr_t clientdata, caddr_t calldata)
 {
-   Window nullwin;
-   int    nullint, xpos, ypos;
    XPoint newpos;
-   u_int   nullui;
    polyptr      newbox;
    pointlist	pointptr;
 
    newbox = TOPOLY(EDITPART);
-   XQueryPointer(dpy, win, &nullwin, &nullwin, &nullint, &nullint, &xpos, &ypos,
-	&nullui);
+   newpos = UGetCursorPos();
+   u2u_snap(&newpos);
 
-   snap(xpos, ypos, &newpos);
    if (areastruct.save.x == newpos.x && areastruct.save.y == newpos.y) return;
 
    UDrawPolygon(areastruct.topobject, newbox);
@@ -894,16 +882,12 @@ void trackbox(Widget w, caddr_t clientdata, caddr_t calldata)
 
 void trackwire(Widget w, caddr_t clientdata, caddr_t calldata)
 {
-   Window nullwin;
-   int    nullint, xpos, ypos;
    XPoint newpos, *tpoint;
-   u_int   nullui;
    polyptr	newwire;
 
    newwire = TOPOLY(EDITPART);
-   XQueryPointer(dpy, win, &nullwin, &nullwin, &nullint, &nullint, &xpos, &ypos,
-	&nullui);
-   snap(xpos, ypos, &newpos);
+   newpos = UGetCursorPos();
+   u2u_snap(&newpos);
    if (areastruct.manhatn) manhattanize(&newpos, newwire);
 
    if (areastruct.save.x != newpos.x || areastruct.save.y != newpos.y) {
@@ -1033,18 +1017,15 @@ void finddir(polyptr lastpoly)
 
 void trackpoly(Widget w, caddr_t clientdata, caddr_t calldata)
 {
-   Window nullwin;
-   int    nullint, xpos, ypos;
    XPoint newpos, *curpt;
-   u_int   nullui;
    polyptr      newpoly;
+   int		nullint;
 
    newpoly = (eventmode == EPATH_MODE) ?
 	     (polyptr)(*((*((pathptr *)EDITPART))->plist
 	     + areastruct.editsubpart)) : TOPOLY(EDITPART);
-   XQueryPointer(dpy, win, &nullwin, &nullwin, &nullint, &nullint, &xpos, &ypos,
-	&nullui);
-   snap(xpos, ypos, &newpos);
+   newpos = UGetCursorPos();
+   u2u_snap(&newpos);
    if (areastruct.save.x == newpos.x && areastruct.save.y == newpos.y) return;
 
    UDrawPolygon(areastruct.topobject, newpoly);
