@@ -6,12 +6,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 #include <sys/types.h>
 
-#if defined(linux) || defined(SVR4)
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #include <unistd.h>
 #define direct dirent
@@ -31,6 +33,7 @@
 
 #include "Xw/Xw.h"
 #include "Xw/WorkSpace.h"
+#include "Xw/TextEdit.h"
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -61,6 +64,8 @@ extern ApplicationData appdata;
 extern int 	  *appcolors;
 extern short	  popups;     /* total number of popup windows */
 extern char	  _STR2[250];
+extern char	  _STR[150];
+extern Globaldata xobjs;
 
 Pixmap   flistpix = (Pixmap)NULL;    /* For file-selection widget */
 short    flstart, flfiles, flcurrent;
@@ -151,6 +156,61 @@ void endfiletrack(Widget w, caddr_t clientdata, XCrossingEvent *event)
    XSetFunction(dpy, areastruct.gc, areastruct.gctype);
 }
 
+/*----------------------------------------------------------------------*/
+/* Crash recovery:  load the file, and link the tempfile name to it.    */
+/*----------------------------------------------------------------------*/
+
+void crashrecover()
+{
+   if (xobjs.tempfile != NULL) {
+      unlink(xobjs.tempfile);
+      free(xobjs.tempfile);
+   }
+   xobjs.tempfile = strdup(_STR2);
+
+   startloadfile();
+}
+
+/*----------------------------------------------------------------------*/
+/* Look for any files left over from a crash.                           */
+/*----------------------------------------------------------------------*/
+
+void findcrashfiles()
+{
+   DIR *cwd;
+   struct direct *dp;
+   struct stat sbuf;
+   uid_t userid = getuid();
+   time_t recent = 0;
+
+   cwd = opendir("/tmp");
+   if (cwd == NULL) return;	/* No tmp directory, no tmp files! */
+
+   while ((dp = readdir(cwd)) != NULL) {
+      sprintf(_STR, "/tmp/%s", dp->d_name);
+      if (!strncmp(_STR + 5, "XC", 2)) {
+         stat(_STR, &sbuf);
+         if (sbuf.st_uid == userid) {
+	    if ((recent == 0) || (sbuf.st_ctime > recent)) {
+	       recent = sbuf.st_ctime;
+	       strcpy(_STR2, _STR);
+	    }
+	 }
+      }
+   }
+   closedir(cwd);
+   
+   if (recent > 0) {	/* There exists at least one temporary file */
+			/* belonging to this user.  Ask to recover  */
+			/* the most recent one.			    */
+
+      /* Warn user of existing tempfile, and ask user if file	*/
+      /* should be recovered immediately.			*/
+
+      getfile(NULL, Number(RECOVER), NULL);   /* Crash recovery mode */
+   }
+}  
+
 /*-------------------------------------------------------------------------*/
 /* Make a list of the files in the list widget window			   */
 /*-------------------------------------------------------------------------*/
@@ -179,7 +239,7 @@ void listfiles(Widget w, caddr_t clientdata, caddr_t calldata)
    XtnSetArg(XtNheight, &textheight);
    XtGetValues(w, wargs, n);
 
-   /* Generate a new flistpix pixmap if currently nonexistant */
+   /* Generate a new flistpix pixmap if currently nonexistent */
 
    if (!flistpix) {
 
@@ -496,7 +556,11 @@ void genfilelist(Widget parent, Widget entertext, Dimension width)
 
 int lookdirectory(char *lstring)
 {
-   int slen = strlen(_STR2);
+   int slen;
+
+   tilde_expand(lstring);
+   slen = strlen(lstring);
+
    if (lstring[slen - 1] == '/' || (opendir(lstring) != NULL)) {
       if (lstring[slen - 1] != '/') strcat(lstring, "/");
       cwdname = (char *)realloc(cwdname, (slen + 2) * sizeof(char));
