@@ -5,8 +5,11 @@
    
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 
@@ -37,6 +40,8 @@ extern fontinfo *fonts;
 extern Globaldata xobjs;
 extern short beeper;
 
+extern float version;
+
 /*----------------------------------------------------------------------*/
 /* Find the file containing the encoding for the given font		*/
 /*----------------------------------------------------------------------*/
@@ -104,7 +109,7 @@ FILE *findfontfile(char *fontname)
 	    /* font on the supposition that this font is derived from the */
 	    /* root font.						  */
 
-	    strcpy(tempname, fontname);
+	    strncpy(tempname, fontname, 99);
 	    if ((dashptr = strrchr(tempname, '-')) != NULL) {
 	       *dashptr = '\0';
 	       if ((fd = findfontfile(tempname)) != NULL) return fd;
@@ -158,6 +163,7 @@ void loadfontfile(char *fontname)
    float fontscale = 1.0;
    objectptr *j, *eptr;
    objectptr *encoding = NULL;
+   float saveversion = version;
 
    /* check to see if font name is in list of fonts */
 
@@ -171,7 +177,7 @@ void loadfontfile(char *fontname)
 
    while (fgets(temp, 249, fd) != NULL) {
       if (*temp == '\n') continue;
-      sscanf(temp, "%s", commandstr);
+      sscanf(temp, "%29s", commandstr);
       for(cmdptr = commandstr; isspace(*cmdptr); cmdptr++);
 
       /* very liberal about comment line characters */
@@ -179,7 +185,7 @@ void loadfontfile(char *fontname)
       if (*cmdptr == '#' || *cmdptr == '%' || *cmdptr == ';');
 
       else if (!strcmp(cmdptr, "name:")) {
-	 sscanf(temp, "%*s %s", tempname);
+	 sscanf(temp, "%*s %99s", tempname);
 
          /* If font is already in list, ignore it 			   */
 	 /* This condition should be caught at the top of the routine. . . */
@@ -196,25 +202,31 @@ void loadfontfile(char *fontname)
 
       else if (!strcmp(cmdptr, "file:") || !strcmp(cmdptr, "load:")) {
 	 /* printf("found file identifier in xfe file\n"); */
-	 sscanf(temp, "%*s %s", _STR);
+	 sscanf(temp, "%*s %149s", _STR);
+
+	 /* since we can be in the middle of a file load, protect the	*/
+	 /* current version number for the file load.			*/
+
+	 version = PROG_VERSION;
 	 loadlibrary(FONTLIB);
+	 version = saveversion;
       }
 
       else if (!strcmp(cmdptr, "family:")){
-	 sscanf(temp, "%*s %s", tempname);
+	 sscanf(temp, "%*s %99s", tempname);
 	 family = (char *)malloc((1 + strlen(tempname)) * sizeof(char));
 	 strcpy(family, tempname);
       }
 
       else if (!strcmp(cmdptr, "weight:")){
-	 sscanf(temp, "%*s %s", tempname);
+	 sscanf(temp, "%*s %99s", tempname);
 	 tempname[0] = tolower(tempname[0]);
 	 if (!strcmp(tempname, "bold"))
 	    flags |= 0x01;
       }
 
       else if (!strcmp(cmdptr, "shape:")){
-	 sscanf(temp, "%*s %s", tempname);
+	 sscanf(temp, "%*s %99s", tempname);
 	 tempname[0] = tolower(tempname[0]);
 	 if (!strcmp(tempname, "italic") || !strcmp(tempname, "oblique")
 		|| !strcmp(tempname, "slanted"))
@@ -226,7 +238,7 @@ void loadfontfile(char *fontname)
       }
 
       else if (!strcmp(cmdptr, "type:")) {
-	 sscanf(temp, "%*s %s", tempname);
+	 sscanf(temp, "%*s %99s", tempname);
 	 tempname[0] = tolower(tempname[0]);
 	 if (!strcmp(tempname, "drawn") || !strcmp(tempname, "vectored"))
 	    flags |= 0x08;
@@ -238,7 +250,7 @@ void loadfontfile(char *fontname)
 	 }
 	 else {
 	    char *psname2;
-	    sscanf(temp, "%*s %s", tempname);
+	    sscanf(temp, "%*s %99s", tempname);
 	    psname2 = (char *)malloc((1 + strlen(tempname)) * sizeof(char));
 	    strcpy(psname2, tempname);
 	    flags &= 0xffe0;		/* shares these flags with original */
@@ -246,17 +258,17 @@ void loadfontfile(char *fontname)
 
 	    /* determine rest of flags */
 
-	    sscanf(temp, "%*s %*s %s", tempname);
+	    sscanf(temp, "%*s %*s %99s", tempname);
 	    tempname[0] = tolower(tempname[0]);
 	    if (!strcmp(tempname, "bold"))
 	       flags |= 0x1;
 
-	    sscanf(temp, "%*s %*s %*s %s", tempname);
+	    sscanf(temp, "%*s %*s %*s %99s", tempname);
 	    tempname[0] = tolower(tempname[0]);
 	    if (!strcmp(tempname, "italic") || !strcmp(tempname, "oblique"))
 	       flags |= 0x2;
 
-	    sscanf(temp, "%*s %*s %*s %*s %s", tempname);
+	    sscanf(temp, "%*s %*s %*s %*s %99s", tempname);
 	    tempname[0] = tolower(tempname[0]);
 	    if (!strcmp(tempname, "drawn") || !strcmp(tempname, "vectored"))
 	       flags |= 0x08;
@@ -279,7 +291,7 @@ void loadfontfile(char *fontname)
       }
 
       else if (!strcmp(cmdptr, "encoding:")) {
-	 sscanf(temp, "%*s %s", tempname);
+	 sscanf(temp, "%*s %99s", tempname);
 	 
 	 if (!strcmp(tempname, "special") || !strcmp(tempname, "Special")) {
 	    flags |= 0x80;
@@ -309,7 +321,7 @@ void loadfontfile(char *fontname)
 	    char *temp2 = temp;
 	    while (*temp2 != '\0') {
 	       if ((int)(eptr - encoding) == 256) break;
-	       sscanf(temp2, "%s", tempname);
+	       sscanf(temp2, "%99s", tempname);
 	       *eptr = (objectptr) NULL;
 	       for (j = xobjs.fontlib.library; j < xobjs.fontlib.library +
 			xobjs.fontlib.number; j++) {
