@@ -6,9 +6,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
+#include <ctype.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 
@@ -64,7 +67,7 @@ void loadlgf(int mode)
    genericptr *iolabel;
    int i, sigs;
 
-   sscanf(_STR, "%s", inname);
+   sscanf(_STR, "%149s", inname);
 
    ps = fopen(inname, "r");
    if (ps == NULL) {
@@ -192,7 +195,7 @@ void loadlgf(int mode)
       	          for (lineptr = temp; isspace(*lineptr) && *lineptr != '\n'; lineptr++);
 	       } while (*lineptr == '\n');
 	       
-	       sscanf(lineptr, "%hd %s", &signets[i], tmpstring);
+	       sscanf(lineptr, "%hd %249s", &signets[i], tmpstring);
 
 	       signals[i] = (char *)malloc((strlen(tmpstring) + 1) * sizeof(char));
 	       sprintf(signals[i], "%s", tmpstring);
@@ -552,7 +555,7 @@ void loadmat4(caddr_t nullval)
    float curwidth = 1.0;
    int tmpstyle = UNCLOSED;
 
-   sscanf(_STR, "%s", inname);
+   sscanf(_STR, "%149s", inname);
 
    ps = fopen(inname, "r");
    if (ps == NULL) {
@@ -629,7 +632,7 @@ void loadmat4(caddr_t nullval)
       if (lineptr != buffer && *buffer != '%') {
          for (keyptr = lineptr - 1; isspace(*keyptr) && keyptr != buffer; keyptr--);
          for (; !isspace(*keyptr) && keyptr != buffer; keyptr--);
-         sscanf(keyptr, "%s", keyword);
+         sscanf(keyptr, "%29s", keyword);
 
 	 if (!strcmp(keyword, "showpage")) {
             free(buffer);
@@ -646,7 +649,7 @@ void loadmat4(caddr_t nullval)
 	       addnewcolorentry(curcolor);	
 	    }
 	    if ((bb = strchr(buffer, '/')) != NULL) {
-	       sscanf(bb, "%s", &colorstr[matcolors]);
+	       sscanf(bb, "%4s", &colorstr[matcolors]);
 	       matcolors++;
 	    }
 	 } 
