@@ -10,7 +10,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 #include <math.h>
@@ -467,10 +469,45 @@ short UTopTransScale(float length)
    return (short)(length * UTopScale() + 0.45);
 }
 
-/*------------------------------------------------------------------------*/
-/* Translate a point to the nearest snap-to grid point			  */
-/*------------------------------------------------------------------------*/
-/* user coordinates to user coordinates version 			  */
+/*----------------------------------------------------------------------*/
+/* Get the cursor position						*/
+/*----------------------------------------------------------------------*/
+
+XPoint UGetCursor()
+{
+   Window nullwin;
+   int    nullint, xpos, ypos;
+   u_int   nullui;
+   XPoint newpos;
+ 
+   XQueryPointer(dpy, win, &nullwin, &nullwin, &nullint, &nullint, &xpos, &ypos,
+        &nullui);
+
+   newpos.x = xpos;
+   newpos.y = ypos;
+
+   return newpos;
+}
+
+/*----------------------------------------------------------------------*/
+/* Get the cursor position and translate to user coordinates		*/
+/*----------------------------------------------------------------------*/
+
+XPoint UGetCursorPos()
+{
+   XPoint winpos, userpos;
+ 
+   winpos = UGetCursor();
+
+   window_to_user(winpos.x, winpos.y, &userpos);  
+
+   return userpos;
+}
+
+/*----------------------------------------------------------------------*/
+/* Translate a point to the nearest snap-to grid point			*/
+/*----------------------------------------------------------------------*/
+/* user coordinates to user coordinates version 			*/
 
 void u2u_snap(XPoint *uvalue)
 {
@@ -934,7 +971,7 @@ void calcbboxsingle(genericptr *bboxgen, short *llx, short *lly, short *urx,
 #endif
 									)
 {
-   XPoint points[4], npoints[4];
+   XPoint npoints[4];
    short j;
 
    /* For each screen element, compute the extents and revise bounding	*/
@@ -1323,26 +1360,49 @@ void UDrawLine(objinstptr localdata, XPoint *pt1, XPoint *pt2)
    UDrawSimpleLine(localdata, pt1, pt2); 
 } 
 
-/*-------------------------------------------------------------------------*/
-/* Add "X" at string origin						   */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Add "X" at string origin						*/
+/*----------------------------------------------------------------------*/
+
+void UDrawXAt(XPoint *upt)
+{
+   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   XDrawLine(dpy, win, areastruct.gc, upt->x - 3, upt->y - 3, upt->x + 3, upt->y + 3);
+   XDrawLine(dpy, win, areastruct.gc, upt->x + 3, upt->y - 3, upt->x - 3, upt->y + 3);
+}
+
+/*----------------------------------------------------------------------*/
+/* Draw "X" on current level						*/
+/*----------------------------------------------------------------------*/
 
 void UDrawX(labelptr curlabel)
 {
    XPoint upt;
 
    user_to_window(curlabel->position, &upt);
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
-   XDrawLine(dpy, win, areastruct.gc, upt.x - 3, upt.y - 3, upt.x + 3, upt.y + 3);
-   XDrawLine(dpy, win, areastruct.gc, upt.x + 3, upt.y - 3, upt.x - 3, upt.y + 3);
+   UDrawXAt(&upt);
+}
+
+#ifdef SCHEMA
+
+/*----------------------------------------------------------------------*/
+/* Draw "X" on top level (only for LOCAL and GLOBAL pin labels)		*/
+/*----------------------------------------------------------------------*/
+
+void UDrawXDown(labelptr curlabel)
+{
+   XPoint upt;
+
+   if (areastruct.pinpointon && curlabel->pin && curlabel->pin != INFO) {
+      UTransformbyCTM(DCTM, &curlabel->position, &upt, 1);
+      UDrawXAt(&upt);
+   }
 }
 
 /*-------------------------------------------------------------------------*/
 /* Adjust a pinlabel position to account for pad spacing		   */
 /*-------------------------------------------------------------------------*/
 
-#ifdef SCHEMA
-
 void pinadjust (short justify, short *xpoint, short *ypoint, short dir)
 {
    int delx, dely;
@@ -1643,7 +1703,7 @@ void UDrawObject(objinstptr localdata, objectptr theobject, short level,
    int		defaultcolor = passcolor;
    int		curcolor = passcolor;
    XPoint 	bboxin, bboxout[2];
-   Boolean	xm, ym;
+   u_char	xm, ym;
    objectpair	*newhierarchy, *lasthierarchy;
 
    /* add self to hierarchy list */
@@ -1669,7 +1729,7 @@ void UDrawObject(objinstptr localdata, objectptr theobject, short level,
    ym = (bboxout[0].y < bboxout[1].y) ? 0 : 1;  
 
    if (bboxout[xm].x < areastruct.width && bboxout[ym].y < areastruct.height &&
-       bboxout[!xm].x > 0 && bboxout[!ym].y > 0) {       
+       bboxout[1 - xm].x > 0 && bboxout[1 - ym].y > 0) {       
 
      /* draw all of the elements */
    
@@ -1718,6 +1778,10 @@ void UDrawObject(objinstptr localdata, objectptr theobject, short level,
 	     if (level == 0 || TOLABEL(areagen)->pin == False)
 #endif
              UDrawString(localdata, TOLABEL(areagen), curcolor);
+#ifdef SCHEMA
+	     else if (level == 1 && TOLABEL(areagen)->pin == True)
+		UDrawXDown(TOLABEL(areagen));
+#endif
 	     break;
        }
      }
