@@ -8,6 +8,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -66,95 +67,96 @@ void printhelppix(Widget w)
 {
    static char *helptitle = "Macro Key Binding Summary:";
    static helpstruct helptext[] = {
-    XCF_Zoom_In,	"Zoom in 3/2",
-    XCF_Zoom_Out,	"Zoom out 3/2",
-    XCF_Pan,		"Pan to center",
-    XCF_Pan_Up,		"Half-page pan Up",
-    XCF_Pan_Down,	"Half-page pan Down",
-    XCF_Pan_Right,	"Half-page pan Right",
-    XCF_Pan_Left,	"Half-page pan Left",
-    XCF_Double_Snap,	"Double snap-to spacing",
-    XCF_Halve_Snap,	"Halve snap-to spacing",
-    XCF_Next_Library,	"Go To Next Library",
-    XCF_Push,		"Push object",
-    XCF_Pop,		"Pop object",
-    XCF_Redraw,		"Refresh screen",
-    XCF_Page,		"Go To Page",
-    XCF_Write,		"Popup Output dialog",
+    { XCF_Zoom_In,	"Zoom in 3/2"},
+    { XCF_Zoom_Out,	"Zoom out 3/2"},
+    { XCF_Pan,		"Pan to center"},
+    { XCF_Pan_Up,	"Half-page pan Up"},
+    { XCF_Pan_Down,	"Half-page pan Down"},
+    { XCF_Pan_Right,	"Half-page pan Right"},
+    { XCF_Pan_Left,	"Half-page pan Left"},
+    { XCF_Double_Snap,	"Double snap-to spacing"},
+    { XCF_Halve_Snap,	"Halve snap-to spacing"},
+    { XCF_Next_Library,	"Go To Next Library"},
+    { XCF_Push,		"Push object"},
+    { XCF_Pop,		"Pop object"},
+    { XCF_Redraw,	"Refresh screen"},
+    { XCF_Page,		"Go To Page"},
+    { XCF_Write,	"Popup Output dialog"},
 #ifdef HAVE_PYTHON
-    XCF_Prompt,		"Python Command entry",
+    { XCF_Prompt,	"Python Command entry"},
 #else
-    XCF_Prompt,		"Command entry",
+    { XCF_Prompt,	"Command entry"},
 #endif
-    XCF_Exit,		"Quit XCircuit",
-    XCF_SPACER,		NULL,
-    XCF_Wire,		"Begin Polygon",
-    XCF_Box,		"Begin Box",
-    XCF_Arc,		"Begin Arc",
-    XCF_Text,		"Begin Text",
-    XCF_Spline,		"Begin Spline",
-    XCF_Select_Save,	"Make object",
-    XCF_Join,		"Join elements (make path)",
-    XCF_Unjoin,		"Un-join elements",
-    XCF_Dot,		"Place a dot",
-    XCF_SPACER,		NULL,
-    XCF_Delete,		"Delete",
-    XCF_Undelete,	"Undelete",
-    XCF_Select,		"Select",
-    XCF_Unselect,	"Deselect",
-    XCF_Copy,		"Copy",
-    XCF_Edit,		"Edit",
-    XCF_SPACER,		NULL,
-    XCF_Rotate,		"Rotate",
-    XCF_Flip_X,		"Flip horizontally",
-    XCF_Flip_Y,		"Flip vertically",
-    XCF_Snap,		"Snap to grid",
-    XCF_Attach,		"Attach to",
-    XCF_Dashed,		"Dashed line style",
-    XCF_Dotted,		"Dotted line style",
-    XCF_Solid,		"Solid line style",
-    XCF_SPACER,		NULL,
-    XCF_Justify,	"Text Justification",
-    XCF_Superscript,	"Text Superscript",
-    XCF_Subscript,	"Text Subscript",
-    XCF_Nextfont,	"Toggle text font",
-    XCF_Boldfont,	"Begin Bold text",
-    XCF_Italicfont,	"Begin Italic text",
-    XCF_Normalfont,	"Resume normal text",
-    XCF_ISO_Encoding,	"Begin Latin-1 encoding",
-    XCF_Overline,	"Begin text overline",
-    XCF_Underline,	"Begin text underline",
-    XCF_Parameter,	"Insert parameter",
-    XCF_Halfspace,	"Insert half-space",
-    XCF_Quarterspace,	"Insert quarter-space",
-    XCF_Return_Char,	"Insert return character",
-    XCF_Special,	"Insert special character",
-    XCF_TabStop,	"Set tab stop",
-    XCF_TabForward,	"Forward tab",
-    XCF_TabBackward,	"Backward tab",
-    XCF_Text_Home,	"Go to label beginning",
-    XCF_Text_End,	"Go to label end",
-    XCF_Text_Left,	"Move left one position",
-    XCF_Text_Right,	"Move right one position",
-    XCF_Text_Delete,	"Delete character(s)",
-    XCF_Text_Return,	"End text edit",
-    XCF_SPACER,		NULL,
-    XCF_Library_Edit,	"Library name edit",
-    XCF_Library_Move,	"Library move object/page",
-    XCF_Library_Delete,	"Library object delete",
-    XCF_Library_Hide,	"Library hide object",
-    XCF_Library_Duplicate, "Library copy object",
+    { XCF_Exit,		"Quit XCircuit"},
+    { XCF_SPACER,	NULL},
+    { XCF_Wire,		"Begin Polygon"},
+    { XCF_Box,		"Begin Box"},
+    { XCF_Arc,		"Begin Arc"},
+    { XCF_Text,		"Begin Text"},
+    { XCF_Spline,	"Begin Spline"},
+    { XCF_Select_Save,	"Make object"},
+    { XCF_Join,		"Join elements (make path)"},
+    { XCF_Unjoin,	"Un-join elements"},
+    { XCF_Dot,		"Place a dot"},
+    { XCF_SPACER,	NULL},
+    { XCF_Delete,	"Delete"},
+    { XCF_Undelete,	"Undelete"},
+    { XCF_Select,	"Select"},
+    { XCF_Unselect,	"Deselect"},
+    { XCF_Copy,		"Copy"},
+    { XCF_Edit,		"Edit"},
+    { XCF_SPACER,	NULL},
+    { XCF_Rotate,	"Rotate"},
+    { XCF_Flip_X,	"Flip horizontally"},
+    { XCF_Flip_Y,	"Flip vertically"},
+    { XCF_Snap,		"Snap to grid"},
+    { XCF_Attach,	"Attach to"},
+    { XCF_Dashed,	"Dashed line style"},
+    { XCF_Dotted,	"Dotted line style"},
+    { XCF_Solid,	"Solid line style"},
+    { XCF_SPACER,	NULL},
+    { XCF_Justify,	"Text Justification"},
+    { XCF_Superscript,	"Text Superscript"},
+    { XCF_Subscript,	"Text Subscript"},
+    { XCF_Nextfont,	"Toggle text font"},
+    { XCF_Boldfont,	"Begin Bold text"},
+    { XCF_Italicfont,	"Begin Italic text"},
+    { XCF_Normalfont,	"Resume normal text"},
+    { XCF_ISO_Encoding,	"Begin Latin-1 encoding"},
+    { XCF_Overline,	"Begin text overline"},
+    { XCF_Underline,	"Begin text underline"},
+    { XCF_Parameter,	"Insert parameter"},
+    { XCF_Halfspace,	"Insert half-space"},
+    { XCF_Quarterspace,	"Insert quarter-space"},
+    { XCF_Return_Char,	"Insert return character"},
+    { XCF_Special,	"Insert special character"},
+    { XCF_TabStop,	"Set tab stop"},
+    { XCF_TabForward,	"Forward tab"},
+    { XCF_TabBackward,	"Backward tab"},
+    { XCF_Text_Home,	"Go to label beginning"},
+    { XCF_Text_End,	"Go to label end"},
+    { XCF_Text_Left,	"Move left one position"},
+    { XCF_Text_Right,	"Move right one position"},
+    { XCF_Text_Delete,	"Delete character(s)"},
+    { XCF_Text_Return,	"End text edit"},
+    { XCF_SPACER,	NULL},
+    { XCF_Library_Edit,	"Library name edit"},
+    { XCF_Library_Move,	"Library move object/page"},
+    { XCF_Library_Delete, "Library object delete"},
+    { XCF_Library_Hide,	"Library hide object"},
+    { XCF_Library_Duplicate, "Library copy object"},
 #ifdef SCHEMA
-    XCF_SPACER,		NULL,
-    XCF_Pin_Label,	"Make Pin Label",
-    XCF_Info_Label,	"Make Info Label",
-    XCF_Swap,		"Go to Symbol or Schematic",
-    XCF_Connectivity,	"See net connectivity",
-    XCF_Sim,		"Generate Sim netlist",
-    XCF_SPICE,		"Generate SPICE netlist",
-    XCF_PCB,		"Generate PCB netlist",
+    { XCF_SPACER,	NULL},
+    { XCF_Pin_Label,	"Make Pin Label"},
+    { XCF_Info_Label,	"Make Info Label"},
+    { XCF_Swap,		"Go to Symbol or Schematic"},
+    { XCF_Connectivity,	"See net connectivity"},
+    { XCF_Sim,		"Generate Sim netlist"},
+    { XCF_SPICE,	"Generate SPICE netlist"},
+    { XCF_SPICEflat,	"Generate flattened SPICE"},
+    { XCF_PCB,		"Generate PCB netlist"},
 #endif
-    XCF_ENDDATA,	NULL,
+    { XCF_ENDDATA,	NULL},
     };
 
     XGCValues	values;
