@@ -10,7 +10,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#include <ctype.h>	 /* for tolower(), toupper() */
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 #include <math.h>
@@ -83,7 +86,7 @@ static char *function_names[NUM_FUNCTIONS] = {
 
 #ifdef SCHEMA
    , "Netlist", "Swap", "Pin Label", "Info Label", "Connectivity",
-   "Sim", "SPICE", "PCB"
+   "Sim", "SPICE", "PCB", "SPICE Flat"
 #endif
 };
 
@@ -197,11 +200,11 @@ int string_to_func(const char *funcstring, short *value)
 
 int string_to_key(const char *keystring)
 {
-   int ct, bnum, keywstate = 0;
+   int ct, keywstate = 0;
    const char *kptr = keystring;
 
    while(1) {
-      if (*kptr == '\0') return;
+      if (*kptr == '\0') return -1;
       if (!strncmp(kptr, "XK_", 3))
 	 kptr += 3;
       else if (!strncmp(kptr, "Shift_", 6)) {
@@ -224,7 +227,8 @@ int string_to_key(const char *keystring)
 	 keywstate |= ALT;
 	 kptr += 5;
       }
-      else if (*kptr++ == '^') {
+      else if (*kptr == '^') {
+	 kptr++;
 	 ct = (int)tolower(*kptr);
 	 keywstate |= CTRL | ct;
 	 break;
@@ -386,13 +390,13 @@ void remove_keybinding(const char *keystring, const char *fstring)
 /* Add a key binding to the list				*/
 /*--------------------------------------------------------------*/
 
-void add_vbinding(int keywstate, int function, short value)
+int add_vbinding(int keywstate, int function, short value)
 {
    keybinding *newbinding;
 
    /* If key is already bound to the function, ignore it */
 
-   if (isbound(keywstate, function)) return;
+   if (isbound(keywstate, function)) return 1;
 
    /* Add the new key binding */
 
@@ -402,34 +406,32 @@ void add_vbinding(int keywstate, int function, short value)
    newbinding->value = value;
    newbinding->nextbinding = keylist;
    keylist = newbinding;
+   return 0;
 }
 
 /*--------------------------------------------------------------*/
 /* Wrapper function for key binding without any values		*/
 /*--------------------------------------------------------------*/
 
-void add_binding(int keywstate, int function)
+int add_binding(int keywstate, int function)
 {
-   add_vbinding(keywstate, function, -1);
+   return add_vbinding(keywstate, function, -1);
 }
 
 /*--------------------------------------------------------------*/
 /* Wrapper function for key binding with function as string	*/
 /*--------------------------------------------------------------*/
 
-void add_keybinding(const char *keystring, const char *fstring)
+int add_keybinding(const char *keystring, const char *fstring)
 {
    short value = -1;
    int function = string_to_func(fstring, &value);
    int keywstate = string_to_key(keystring);
 
-   if (function < 0) {
-      sprintf(_STR2, "Function \"%s\" is not known.", fstring);
-      Wprintf(_STR2);
-      return;
-   }
+   if (function < 0)
+      return -1;
    else
-      add_vbinding(keywstate, function, value);
+      return add_vbinding(keywstate, function, value);
 }
 
 /*--------------------------------------------------------------*/
@@ -478,7 +480,9 @@ void default_keybindings()
    add_binding(XK_End, XCF_Text_End);
    add_binding(XK_Tab, XCF_TabForward);
    add_binding(SHIFT | XK_Tab, XCF_TabBackward);
+#ifdef XK_ISO_Left_Tab
    add_binding(SHIFT | XK_ISO_Left_Tab, XCF_TabBackward);
+#endif
    add_binding(ALT | XK_Tab, XCF_TabStop);
    add_binding(XK_KP_Add, XCF_Superscript);
    add_binding(XK_KP_Subtract, XCF_Subscript);
@@ -574,6 +578,7 @@ void default_keybindings()
    add_binding(ALT | XK_w, XCF_Connectivity);
    add_binding(ALT | XK_s, XCF_Sim);
    add_binding(SHIFT | ALT | XK_S, XCF_SPICE);
+   add_binding(ALT | XK_f, XCF_SPICEflat);
    add_binding(ALT | XK_p, XCF_PCB);
 #endif
 
