@@ -10,7 +10,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 
@@ -321,9 +323,12 @@ void composepagelib(short mode)
    objectptr libobj, libinst = xobjs.libtop[mode];
    short i;
    polyptr *drawbox;
+   labelptr *pagelabel;
+   stringpart *strptr;
    pointlist pointptr;
    int bbuf, xdel, ydel, gxsize, gysize;
    int pages = (mode == PAGELIB) ? xobjs.pages : xobjs.numlibs;
+   short fval = findhelvetica();
 
    reset(libinst, NORMAL);
 
@@ -338,14 +343,13 @@ void composepagelib(short mode)
    for (i = 0; i < pages; i++) {
       libobj = (mode == PAGELIB) ? xobjs.pagelist[i]->pageobj
 		: xobjs.libtop[i + LIBRARY];
-      if (libobj == NULL) continue;
-      drawinst = makeinstofpage(libinst, libobj);
-      pageinstpos(mode, i, drawinst, gxsize, gysize, xdel, ydel);
-   }
+      if (libobj != NULL) {
+         drawinst = makeinstofpage(libinst, libobj);
+         pageinstpos(mode, i, drawinst, gxsize, gysize, xdel, ydel);
+      }
 
-   /* separate pages (including empty ones) with bounding boxes */
+      /* separate pages (including empty ones) with bounding boxes */
 
-   for (i = 0; i < pages; i++) {
       NEW_POLY(drawbox, libinst);
       (*drawbox)->color = SNAPCOLOR;   /* default red */
       (*drawbox)->style = NORMAL;      /* CLOSED */
@@ -365,6 +369,23 @@ void composepagelib(short mode)
       pointptr->x = (i % gxsize) * xdel + bbuf;
       pointptr->y = -((i / gxsize) + 1) * ydel + bbuf;
       libinst->parts++;
+
+      /* each page gets its name printed at the bottom */
+
+      if (libobj != NULL) {
+	 NEW_LABEL(pagelabel, libinst);
+	 labeldefaults(*pagelabel, False, (pointptr->x + (pointptr-1)->x) / 2,
+		pointptr->y - 5);
+	 (*pagelabel)->color = DEFAULTCOLOR;
+	 (*pagelabel)->scale = 0.75;
+	 (*pagelabel)->string->data.font = fval;
+	 strptr = makesegment(&((*pagelabel)->string), NULL);
+	 strptr->type = TEXT_STRING;
+	 strptr->data.string = (char *) malloc(1 + strlen(libobj->name));
+	 strcpy(strptr->data.string, libobj->name);
+	 (*pagelabel)->justify = TOP | NOTBOTTOM | NOTLEFT;
+	 libinst->parts++;
+      }
    }
 
    /* calculate a bounding box for this display */
@@ -659,7 +680,7 @@ short finddepend(objinstptr libobj, objectptr **compobjp)
 
 void cathide()
 {
-   short *newselect;
+   short i, *newselect;
    objectptr *compobj;
    objinstptr libobj;
 
@@ -684,10 +705,7 @@ void cathide()
    areastruct.selects = 0;
    free(areastruct.selectlist);
 
-   if (objectdata == xobjs.libtop[LIBRARY])
-      composelib(LIBRARY);
-   else
-      composelib(USERLIB);
+   if ((i = is_library(objectdata)) >= 0) composelib(LIBRARY + i);
 
    drawarea(NULL, NULL, NULL);
 }
@@ -758,10 +776,7 @@ void catdelete()
    areastruct.selects = 0;
    free(areastruct.selectlist);
 
-   if (objectdata == xobjs.libtop[LIBRARY])
-      composelib(LIBRARY);
-   else
-      composelib(USERLIB);
+   if ((i = is_library(objectdata)) >= 0) composelib(LIBRARY + i);
 
    drawarea(NULL, NULL, NULL);
 }
@@ -883,10 +898,7 @@ void catmove(XKeyEvent *event)
    }
 
    objectdeselect();
-   if (objectdata == xobjs.libtop[LIBRARY])
-      composelib(LIBRARY);
-   else
-      composelib(USERLIB);
+   if ((i = is_library(objectdata)) >= 0) composelib(LIBRARY + i);
 
    drawarea(NULL, NULL, NULL);
 }
@@ -928,12 +940,31 @@ void copycat()
       (*newobj)->height = oldobj->height;
       (*newobj)->lowerleft.x = oldobj->lowerleft.x;
       (*newobj)->lowerleft.y = oldobj->lowerleft.y;
+      (*newobj)->pcorner.x = oldobj->pcorner.x;
+      (*newobj)->pcorner.y = oldobj->pcorner.y;
+      (*newobj)->viewscale = oldobj->viewscale;
 #ifdef SCHEMA
       /* don't attach the same schematic. . . */
       (*newobj)->symschem = NULL;
-      /* and don't copy the parameter structure (for now) */
-      (*newobj)->params = NULL;
-      (*newobj)->num_params = 0;
+
+      /* Copy the parameter structure */
+      (*newobj)->num_params = oldobj->num_params;
+      (*newobj)->params = (oparamptr *)malloc(oldobj->num_params *
+		sizeof(oparamptr));
+      for (i = 0; i < oldobj->num_params; i++) {
+	 (*newobj)->params[i] = (oparamptr) malloc(sizeof(oparam));
+	 (*newobj)->params[i]->type = oldobj->params[i]->type;
+	 switch (oldobj->params[i]->type) {
+	    case XC_SHORT:
+	       (*newobj)->params[i]->parameter.svalue =
+			oldobj->params[i]->parameter.svalue;
+	       break;
+	    case XC_STRING:
+	       (*newobj)->params[i]->parameter.string =
+			stringcopy(oldobj->params[i]->parameter.string);
+	       break;
+	 }
+      }
 
       (*newobj)->lleft2.x = oldobj->lleft2.x;
       (*newobj)->lleft2.y = oldobj->lleft2.y;
@@ -945,6 +976,7 @@ void copycat()
       (*newobj)->portlist = NULL;
       (*newobj)->calllist = NULL;
       (*newobj)->traversed = 0;
+      (*newobj)->devname = NULL;
 #endif
       (*newobj)->hidden = False;
 
@@ -993,13 +1025,13 @@ void copycat()
 	       } break;
 	    case(ARC): {
 	       register arcptr *narc, carc = TOARC(oldobj->plist + i);
+
 	       NEW_ARC(narc, (*newobj));
 	       arccopy(*narc, carc);
                } break;
       
 	    case(POLYGON): {
 	       register polyptr *npoly, cpoly;
-	       register int j;
 
 	       cpoly = TOPOLY(oldobj->plist + i);
 	       NEW_POLY(npoly, (*newobj));
@@ -1045,7 +1077,7 @@ void copycat()
 
 	       /* Library instance's parameter list is always default (null) */
 	       (*ninst)->params = NULL;
-	      
+
 #ifdef SCHEMA
 	       /* here---deal with attached schematics */
 #endif
