@@ -7,7 +7,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#include <ctype.h>
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 #include <sys/types.h>
@@ -20,6 +23,7 @@
 
 #include "Xw/Xw.h"
 #include "Xw/MenuBtn.h"
+#include "Xw/PopupMgr.h"
 
 /*----------------------------------------------------------------------*/
 /* Local includes							*/
@@ -160,11 +164,13 @@ void getgeneric(buttonsave *saveptr, Widget button, void (*getfunction)(),
    saveptr->buttoncall = getfunction;
    saveptr->dataptr = dataptr;
 
-   XtSetArg(wargs[0], XtNforeground, &saveptr->foreground);
-   XtGetValues(button, wargs, 1);
-   XtSetArg(wargs[0], XtNforeground, OFFBUTTONCOLOR);
-   XtSetValues(button, wargs, 1);
-   XtRemoveAllCallbacks(button, XtNselect);
+   if (button != NULL) {
+      XtSetArg(wargs[0], XtNforeground, &saveptr->foreground);
+      XtGetValues(button, wargs, 1);
+      XtSetArg(wargs[0], XtNforeground, OFFBUTTONCOLOR);
+      XtSetValues(button, wargs, 1);
+      XtRemoveAllCallbacks(button, XtNselect);
+   }
 }
 
 
@@ -512,7 +518,7 @@ void setpagesize(Widget w, XPoint *dataptr)
 
    strcpy(units, "in");
 
-   if (sscanf(_STR2, "%f %*c %f %s", &px, &py, units) < 4) {
+   if (sscanf(_STR2, "%f %*c %f %9s", &px, &py, units) < 4) {
       if (sscanf(_STR2, "%f %*c %f", &px, &py) < 3) {
 	 if ((expos = strchr(_STR2, 'x')) == NULL) {
             Wprintf("Illegal Form for page size.");
@@ -521,7 +527,7 @@ void setpagesize(Widget w, XPoint *dataptr)
 	 else {
 	    *expos = '\0';
 	    if (sscanf(_STR2, "%f", &px) == 0 ||
-		  sscanf(expos + 1, "%f %s", &py, units) == 0) {
+		  sscanf(expos + 1, "%f %9s", &py, units) == 0) {
                Wprintf("Illegal Form for page size.");
 	       return;
 	    }
@@ -958,9 +964,9 @@ void setpagelabel(Widget w, char *dataptr)
    renamepage(areastruct.page);
 }
 
-/*----------------------------------------------------------------*/
-/* Add a new font name to the list of known fonts		  */
-/*----------------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Add a new font name to the list of known fonts		*/
+/*--------------------------------------------------------------*/
 
 void makenewfontbutton()
 {
@@ -1322,14 +1328,6 @@ int createlibrary()
 
 void newpagemenu(Widget w, u_int value, caddr_t nulldata)
 {
-   if (eventmode == CATALOG_MODE) {
-      eventmode = NORMAL_MODE;
-      catreturn();
-   }
-   else if (eventmode != NORMAL_MODE && eventmode != COPY2_MODE) {
-      Wprintf("Cannot switch pages from this mode");
-      return;
-   }
    newpage((short)value);
 }
 
@@ -1384,7 +1382,7 @@ void renamepage(short pagenumber)
       XtSetValues(button, wargs, 1);
    }
    else
-      printf("Error:  No Button Widget named \"%s\"\n", bname);
+      printf("Error:  No Button Widget named \"%9s\"\n", bname);
 }
 
 /*--------------------------------------------------------------*/
@@ -1409,7 +1407,7 @@ void renamelib(short libnumber)
       XtSetValues(button, wargs, 1);
    }
    else
-      printf("Error:  No Button Widget named \"%s\"\n", bname);
+      printf("Error:  No Button Widget named \"%12s\"\n", bname);
 }
 
 /*----------------------------------------------*/
@@ -2174,7 +2172,7 @@ short findbestfont(short curfont, short newfont, short style, short encoding) {
 	 	(!strcmp(fonts[i].family, newfamily))) return i;
       }
       else if (encoding >= 0) {
-	 if ((fonts[i].flags & 0xf80 == newenc) &&
+	 if (((fonts[i].flags & 0xf80) == newenc) &&
 	 	(!strcmp(fonts[i].family, newfamily))) return i;
       }
    }
@@ -2217,7 +2215,7 @@ short findbestfont(short curfont, short newfont, short style, short encoding) {
 void setfontval(Widget w, u_int value, labelptr settext)
 {
    int newfont;
-   short i, tc, tflag;
+   short i, tc;
    stringpart *strptr;
 
    if (settext != NULL) {
@@ -2308,7 +2306,7 @@ void setfont(Widget w, u_int value, caddr_t calldata)
 void setfontstyle(Widget w, u_int value, labelptr settext)
 {
    int newfont;
-   short i, tc, tflag;
+   short i, tc;
    stringpart *strptr;
 
    if (settext != NULL) {
@@ -2398,7 +2396,7 @@ void fontstyle(Widget w, u_int value, caddr_t nulldata)
 void setfontencoding(Widget w, u_int value, labelptr settext)
 {
    int newfont;
-   short i, tc, tflag;
+   short i, tc;
    stringpart *strptr;
 
    if (settext != NULL) {
