@@ -236,11 +236,13 @@ menustruct SchemaMenu[] = {
 	{"Make Global Pin (G)", action(starttext, Number(GLOBAL))},
 	{"Convert Label to...", submenu(PinConvert)},
 /*	{"Make Object Non-Schematic", action(maketrivial, NULL)}, */
-	{"Make Matching Symbol", action(swapschem, Number(1))},
+	{"Make Matching Symbol", action(dobeforeswap, NULL)},
 	{"Associate with Symbol", action(startschemassoc, NULL)},
 	{"Highlight Connectivity", action(connectivity, NULL)},
+	{"Auto-number Components", action(callgennet, Number(4))},
 	{" ", noaction},
 	{"Write spice", action(callgennet, Number(0))},
+	{"Write flattened spice", action(callgennet, Number(3))},
 	{"Write sim", action(callgennet, Number(1))},
 	{"Write pcb", action(callgennet, Number(2))},
 };
@@ -284,6 +286,7 @@ menustruct OptionMenu[] = {
 	{"Alt Colors", action(inversecolor, &areastruct.invert)},
 	{"Show Bounding box", action(toggle, &areastruct.bboxon)},
 #ifdef SCHEMA
+	{"Show Pin Positions", action(toggle, &areastruct.pinpointon)},
 	{"Enable XSchema", action(doxschema, NULL)},
 #endif
 #ifdef HAVE_XPM
@@ -299,7 +302,7 @@ menustruct OptionMenu[] = {
 
 menustruct EditMenu[] = {
 	{"Delete (d)", action(startdelete, NULL)},
-	{"Undelete (u)", action(undelete, Number(DRAW))},
+	{"Undelete (u)", action(xc_undelete, Number(DRAW))},
 	{"Copy (c)", action(startcopy, NULL)},
 	{"Edit (e)", action(startedit, NULL)},
 	{"Rotate/Flip", submenu(RotateMenu)},
