@@ -11,7 +11,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 
@@ -262,7 +264,7 @@ void insertparam()
 {
    labelptr tlab;
    oparamptr ops;
-   short i, j, plen, nparms = 0;
+   short i, nparms = 0;
    int selparm = -1;
 
    /* Don't allow nested parameters */
@@ -317,7 +319,7 @@ void insertparam()
    if ((selparm >= 0) && (selparm < objectdata->num_params))
       labeltext(PARAM_START, (char *)&selparm);
    else
-      Wprintf("Parameter number nonexistant.");
+      Wprintf("Parameter number nonexistent.");
 }
 
 /*--------------------------------------------------------------*/
@@ -327,7 +329,7 @@ void insertparam()
 void makeparam(labelptr thislabel)
 {
    oparamptr ops;
-   stringpart *begpart, *endpart, *strptr;
+   stringpart *begpart, *endpart;
 
    /* cannot form a parameter on a top-level page */
 
@@ -351,8 +353,13 @@ void makeparam(labelptr thislabel)
       begpart = splitstring(textend, &thislabel->string, NULL);
       endpart = splitstring(textpos, &thislabel->string, NULL);
 
-      begpart = makesegment(&thislabel->string, begpart->nextpart);
-      endpart = makesegment(&thislabel->string, endpart->nextpart);
+      /* note that "partial string" may overlap the beginning	*/
+      /* or the end, so deal with these instances.		*/
+
+      begpart = makesegment(&thislabel->string, (begpart->nextpart &&
+		textend > 1) ? begpart->nextpart : begpart);
+      endpart = makesegment(&thislabel->string, (endpart) ?
+		endpart->nextpart : NULL);
    }
    else {				/* full string */
       makesegment(&thislabel->string, thislabel->string);
@@ -371,13 +378,32 @@ void makeparam(labelptr thislabel)
       objectdata->params = (oparamptr *)malloc(sizeof(oparamptr));
    }
    else {
-      objectdata->num_params++;
+      int j, k, p;
+
+      p = objectdata->num_params++;
       objectdata->params = (oparamptr *)realloc(objectdata->params,
 		objectdata->num_params * sizeof(oparamptr));
+
+      /* We have added a parameter.  This requires that all instances	*/
+      /* of the object which do not have NULL params must initialize	*/
+      /* and NULL the pointer to the new parameter.			*/
+
+      for (k = 0; k < xobjs.pages; k++) {
+         initinst(xobjs.pagelist[k]->pageobj, objectdata, p);
+      }
+      for (j = 0; j < xobjs.numlibs; j++) {
+         for (k = 0; k < xobjs.userlibs[j].number; k++) {
+            initinst(*(xobjs.userlibs[j].library + k), objectdata, p);
+         }
+      }
+      for (k = 0; k < xobjs.delbuffer.number; k++) {
+         initinst(*(xobjs.delbuffer.library + k), objectdata, p);
+      }
    }
    *(objectdata->params + objectdata->num_params - 1) = (oparamptr)
 	malloc(sizeof(oparam));
 
+
    ops = *(objectdata->params + objectdata->num_params - 1);
    ops->type = XC_STRING;
    ops->parameter.string = begpart->nextpart;
@@ -387,6 +413,29 @@ void makeparam(labelptr thislabel)
    textend = 0;
 }
 
+/*------------------------------------------------------*/
+/* Search and initialize the selected parameter in all	*/
+/* instances of the specified object.			*/
+/*------------------------------------------------------*/
+
+void initinst(objectptr topobj, objectptr refobj, short pnum)
+{
+   objinstptr tinst;
+   genericptr *pgen;
+
+   if (topobj == NULL) return;
+   for (pgen = topobj->plist; pgen < topobj->plist + topobj->parts; pgen++) {
+      if ((*pgen)->type == OBJECT) {
+	 tinst = TOOBJINST(pgen);
+	 if (tinst->thisobject == refobj) {
+	    if (tinst->params != NULL) {
+	       *(tinst->params + pnum) = NULL;
+	    }
+	 }
+      }
+   }
+}
+
 /*--------------------------------------------------------------*/
 /* Search and destroy the selected parameter in all instances 	*/
 /* of the specified object.					*/
@@ -394,9 +443,9 @@ void makeparam(labelptr thislabel)
 
 void searchinst(objectptr topobj, objectptr refobj, short pnum)
 {
-   oparamptr ops;
+   oparamptr *ops;
    objinstptr tinst;
-   genericptr *pgen, *lgen;
+   genericptr *pgen;
    short k;
 
    if (topobj == NULL) return;
@@ -407,11 +456,11 @@ void searchinst(objectptr topobj, objectptr refobj, short pnum)
 	 if (tinst->thisobject == refobj) {
 	    if (tinst->thisobject->num_params > 0) {
 	       if (tinst->params != NULL) {
-	          ops = *(tinst->params + pnum);
-	          if (ops != NULL)
-		     freelabel(ops->parameter.string);
-		  free(ops);
-		  ops = NULL;
+	          ops = tinst->params + pnum;
+	          if (*ops != NULL)
+		     freelabel((*ops)->parameter.string);
+		  free(*ops);
+		  *ops = NULL;
 		  if (pnum == 0) {
 		     free(tinst->params);
 		     tinst->params = NULL;
@@ -451,8 +500,9 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    labelptr plab;
    oparamptr ops;
    stringpart *strptr, *lastpart, *nextpart, *endpart;
-   stringpart *subs, *newstr;
+   stringpart *subs = NULL, *newstr;
    int p, j, k;
+   Boolean is_last = True;
    
    /* make sure there is a parameter here */
 
@@ -462,8 +512,68 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    }
    p = thispart->data.paramno;
 
+   /* Unparameterizing can cause a change in the string */
+   undrawtext(thislabel);
+
+   /* Now we need to see if there are other places in the object where	*/
+   /* this parameter is used.  If so, we delete only this particular	*/
+   /* instance of the parameter.  Otherwise, we delete the parameter	*/
+   /* and also remove it from the parameters list.			*/
+
+   for (pgen = objectdata->plist; pgen < objectdata->plist + objectdata->parts;
+		pgen++) {
+      if ((*pgen)->type == LABEL) {
+	 plab = TOLABEL(pgen);
+	 for (strptr = plab->string; strptr != NULL; strptr = strptr->nextpart) {
+	    if (strptr->type == PARAM_START) {
+	       if ((strptr != thispart) && (strptr->data.paramno == p)) {
+		  is_last = False;
+		  break;
+	       }
+	    }
+	 }
+	 if (!is_last) break;
+      }
+   }
+
+   ops = *(objectdata->params + p);
+   subs = ops->parameter.string;
+
+   /* Copy the default parameter into the place we are unparameterizing */
+
+   newstr = stringcopy(subs);
+
+   /* Delete the "PARAM_END" off of the copied string and link it into	*/
+   /* the existing string.						*/
+
+   for (endpart = newstr; endpart->nextpart->type != PARAM_END;
+	endpart = endpart->nextpart);
+   free(endpart->nextpart);
+   endpart->nextpart = thispart->nextpart;
+
+   /* Find the stringpart before the parameter call (if any) */
+
+   lastpart = NULL;
+   for (strptr = thislabel->string; strptr != NULL && strptr != thispart;
+		strptr = strptr->nextpart) {
+      lastpart = strptr;
+   }
+   if (lastpart == NULL)
+      thislabel->string = newstr;
+   else
+      lastpart->nextpart = newstr;
+   free(strptr);
+
+   /* Merge strings at boundaries, if possible. */
+   mergestring(lastpart);
+   strptr = lastpart;
+   mergestring(endpart);
+
+   redrawtext(thislabel);
+   if (!is_last) return;
+
    /* Find all instances of this object and remove any parameter */
-   /* substitutions which may have been made.			 */
+   /* substitutions which may have been made.		         */
 
    for (k = 0; k < xobjs.pages; k++) {
       searchinst(xobjs.pagelist[k]->pageobj, objectdata, p);
@@ -476,15 +586,6 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    for (k = 0; k < xobjs.delbuffer.number; k++) {
       searchinst(*(xobjs.delbuffer.library + k), objectdata, p);
    }
-
-   /* Separate the parameter string from the parameter list */
-   
-   ops = *(objectdata->params + p);
-   subs = ops->parameter.string;
-   for (lastpart = subs; lastpart->nextpart->type != PARAM_END;
-		lastpart = lastpart->nextpart);
-   free(lastpart->nextpart);
-   lastpart->nextpart = NULL;
    free(ops);
    
    /* Re-order the remaining parameters */
@@ -499,7 +600,6 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
       objectdata->params = NULL;
    }
 
-   /* Copy the default parameter wherever the parameter is called, and   */
    /* decrement the parameter ID number of all parameters in all strings */
    /* of this object whose parameter IDs are greater.			 */
 
@@ -507,33 +607,10 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
 		pgen++) {
       if ((*pgen)->type == LABEL) {
 	 plab = TOLABEL(pgen);
-	 lastpart = NULL;
-	 strptr = plab->string;
-	 while (strptr != NULL) {
-	    nextpart = strptr->nextpart;
-	    if (strptr->type == PARAM_START) {
+	 for (strptr = plab->string; strptr != NULL; strptr = strptr->nextpart) {
+	    if (strptr->type == PARAM_START)
 	       if (strptr->data.paramno > p)
 		  strptr->data.paramno--;
-	       else if (strptr->data.paramno == p) {
-		  newstr = stringcopy(subs);
-		  for (endpart = newstr; endpart->nextpart != NULL;
-				endpart = endpart->nextpart);
-		  endpart->nextpart = nextpart;
-		  if (lastpart == NULL)
-		     plab->string = newstr;
-		  else
-		     lastpart->nextpart = newstr;
-		  free(strptr);
-
-      		  /* Merge strings at boundaries, if possible. */
-		  mergestring(lastpart);
-		  strptr = lastpart;
-		  mergestring(endpart);
-		  nextpart = strptr->nextpart;
-	       }
-	    }
-	    lastpart = strptr;
-	    strptr = nextpart;
 	 }
       }
    }
@@ -546,9 +623,9 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
 
 void unparameterize(u_int mode)
 {
-   short *fselect, i, ptype = (short)mode;
+   short *fselect, ptype = (short)mode;
    int locpos;
-   stringpart *strptr, *tmpptr;
+   stringpart *strptr, *tmpptr, *lastptr;
    labelptr settext;
 
    if (!checkselect(ptype)) objectselect(ptype);
@@ -563,11 +640,18 @@ void unparameterize(u_int mode)
 	 strptr = strptr->nextpart;
       if (strptr == NULL) return;	/* No parameters */
       tmpptr = settext->string;
-      while (tmpptr != NULL && tmpptr->type != PARAM_START &&
-	 	tmpptr->nextpart != strptr->nextpart)
-	 tmpptr = tmpptr->nextpart;
+      lastptr = NULL;
 
-      if (tmpptr != NULL) unmakeparam(settext, tmpptr);
+      /* Search for parameter boundary, in case selection doesn't include */
+      /* the whole parameter or the parameter start marker.		  */
+
+      for (tmpptr = settext->string; tmpptr != NULL && tmpptr != strptr;
+		tmpptr = nextstringpart(tmpptr, NORMINST))
+	 if (tmpptr->type == PARAM_START) lastptr = tmpptr;
+      /* Finish search, unlinking any parameter we might be inside */
+      for (; tmpptr != NULL; tmpptr = nextstringpart(tmpptr, NORMINST));
+
+      if (lastptr != NULL) unmakeparam(settext, lastptr);
    }
    else {
       for (fselect = areastruct.selectlist; fselect < areastruct.selectlist +
@@ -616,7 +700,6 @@ void parameterize(u_int mode)
 
 Boolean paramcross(objectptr tobj, labelptr tlab)
 {
-   oparamptr ops;
    stringpart *firstptr, *lastptr;
    int locpos;
 
@@ -629,8 +712,9 @@ Boolean paramcross(objectptr tobj, labelptr tlab)
    /* The area between textend and textpos cannot contain a parameter */
    if (textend > 0)
       for (firstptr = findstringpart(textend, &locpos, tlab->string, NORMINST);
-	    firstptr != lastptr; firstptr = firstptr->nextpart)
-         if (firstptr->type == PARAM_START) return True;
+		firstptr != lastptr; firstptr = firstptr->nextpart)
+         if (firstptr->type == PARAM_START || firstptr->type == PARAM_END)
+	    return True;
 
    return False;
 }
@@ -669,7 +753,18 @@ void removeinst(objinstptr thisinst)
 
 void removeparams(objectptr thisobj)
 {
-   int i, j;
+   int i;
+   oparamptr pptr;
+
+   for (i = 0; i < thisobj->num_params; i++) {
+      pptr = thisobj->params[i];
+      if (pptr->type == XC_STRING)
+	 freelabel(pptr->parameter.string);
+      free(pptr);
+   }
+   free(thisobj->params);
+   thisobj->params = NULL;
+   thisobj->num_params = 0;
 }
 
 /*-----------------------------------------------------------------------*/
