@@ -38,6 +38,7 @@ extern void reversefpoints(XfPoint *, short);
 extern void freeparts(short *, short);
 extern void removep(short *, short);
 extern void unjoin();
+extern labelptr findlabelcopy(labelptr, stringpart *);
 extern Boolean neartest(XPoint *, XPoint *);
 extern void join();
 extern void wirebutton(XButtonEvent *);
@@ -101,7 +102,7 @@ extern void startdelete(Widget, caddr_t, caddr_t);
 extern void reviseselect(short *);
 extern void objectdelete(short);
 extern void deletebutton(XButtonEvent *);
-extern short undelete(Widget, u_int, XButtonEvent *);
+extern short xc_undelete(Widget, u_int, XButtonEvent *);
 extern void printname(objectptr);
 extern void checkname(objectptr);
 extern objectptr finddot();
@@ -124,6 +125,8 @@ extern void drawarea(Widget, caddr_t, caddr_t);
 /* from filelist.c: */
 
 extern int fcompare(const void *, const void *);
+extern void crashrecover();
+extern void findcrashfiles();
 extern void dragfilebox(Widget, caddr_t, XMotionEvent *);
 extern void startfiletrack(Widget, caddr_t, XCrossingEvent *);
 extern void endfiletrack(Widget, caddr_t, XCrossingEvent *);
@@ -137,10 +140,12 @@ extern int lookdirectory(char *);
 
 /* from files.c: */
 
+extern void free_single(genericptr *);
 extern void reset(objectptr, short);
 extern void topreset();
 extern void initmem(objectptr);
 extern void freelabel(stringpart *);
+extern Boolean compare_single(genericptr *, genericptr *);
 extern Boolean elemcompare(genericptr *, genericptr *);
 extern short objcompare(objectptr, objectptr);
 extern float getpsscale(float, short);
@@ -170,6 +175,7 @@ extern void findfonts(objectptr, short *);
 extern void savefile(objectptr, short);
 extern int printRGBvalues(char *, int, const char *);
 extern int nosprint(char *);
+extern Boolean tilde_expand(char *);
 extern short writelabel(FILE *, stringpart *, short *);
 extern void writesegment(stringpart *, float *, int *);
 extern void writelabelsegs(FILE *, short *, stringpart *);
@@ -206,6 +212,8 @@ extern void window_to_user(short, short, XPoint *);
 extern void user_to_window(XPoint, XPoint *);
 extern float UTopScale();
 extern short UTopTransScale(float);
+extern XPoint UGetCursor();
+extern XPoint UGetCursorPos();
 extern void u2u_snap(XPoint *);
 extern void snap(short, short, XPoint *);
 extern void UResetCTM(Matrix *);
@@ -219,7 +227,8 @@ extern short flipadjust(short);
 extern void UPreMultCTM(Matrix *, XPoint, float, short);
 extern void UTransformbyCTM(Matrix *, XPoint *, XPoint *, short);
 extern void UfTransformbyCTM(Matrix *, XfPoint *, XPoint *, short);
-extern void UPushCTM();
+extern void UPopCTM(void);
+extern void UPushCTM(void);
 extern void UTransformPoints(XPoint *, XPoint *, short, XPoint, float,
                              short);
 extern void InvTransformPoints(XPoint *, XPoint *, short, XPoint, float,
@@ -282,9 +291,9 @@ extern int string_to_key(const char *);
 extern char *binding_to_string(int, int);
 extern char *func_to_string(int);
 extern char *key_to_string(int);
-extern void add_vbinding(int, int, short);
-extern void add_binging(int, int);
-extern void add_keybinding(const char *, const char *);
+extern int add_vbinding(int, int, short);
+extern int add_binding(int, int);
+extern int add_keybinding(const char *, const char *);
 extern void default_keybindings();
 extern int remove_binding(int, int);
 extern void remove_keybinding(const char *, const char *);
@@ -399,6 +408,7 @@ extern void position_popup(Widget, Widget);
 extern void border_popup(Widget, caddr_t, caddr_t);
 extern void color_popup(Widget, caddr_t, caddr_t);
 extern void fill_popup(Widget, caddr_t, caddr_t);
+extern void dospecial(void);
 
 /* from netlist.c: */
 
@@ -408,21 +418,22 @@ extern void destroynets();
 extern void gennet(char *, char *);
 extern Boolean nonnetwork(polyptr);
 extern int globalmax();
-extern void gennetlist(objectptr);
+extern void gennetlist(objectptr, char *);
 extern void gencalllist(objectptr, Boolean);
 extern void search_on_siblings(objinstptr, objinstptr, objectpair *,
 		short, short, short, short);
 extern int is_resolved(genericptr *, objectpair *, objectptr *);
 extern void highlightnet(objectptr, objinstptr, int);
 extern int pushnetwork(objectpair *, objectptr);
+extern int onsegment(XPoint *, XPoint *, XPoint *);
 extern Boolean neardist(long);
 extern Boolean nearpoint(XPoint *, XPoint *);
 extern int make_tmp_pin(objectptr, XPoint *);
 extern Boolean searchconnect(XPoint *, int, objinstptr);
 extern void addpoly(objectptr, polyptr, int);
 extern long zsign(long, long);
-extern int onsegment(XPoint *, XPoint *, XPoint *, XPoint *, objectptr);
-extern NetlistPtr mergenets(objectptr, int, int);
+extern void mergecleanup(objectptr);
+extern void mergenets(objectptr, int, int);
 extern void removecall(objectptr, CalllistPtr);
 extern void addpin(objectptr, labelptr, int);
 extern void addglobal(labelptr, int);
@@ -434,13 +445,13 @@ extern int porttonet(objectptr, int);
 extern int pointtonet(objectptr, XPoint *);
 extern stringpart *nettopin(int, objectptr, char *);
 extern int pintonet(objectptr, labelptr);
-extern char *parseinfo(objectptr, CalllistPtr, char *, char *);
+extern char *parseinfo(objectptr, CalllistPtr, char *, char *, Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
-extern void writesimflat(objectptr, CalllistPtr, char *, FILE *);
+extern void writeflat(objectptr, CalllistPtr, char *, FILE *, char *);
 extern void writeglobals(objectptr, FILE *);
 extern void writehierarchy(objectptr, CalllistPtr, FILE *);
 extern void writenet(objectptr, char *, char *);
-extern void writepcb(objectptr, CalllistPtr, char *);
+extern void writepcb(objectptr, CalllistPtr, char *, char *);
 extern void outputpcb(objectptr, FILE *);
 extern void freepcb();
 extern void freenets(objectptr);
@@ -458,6 +469,7 @@ extern void unmakeshortp(genericptr *);
 extern void noparmstrcpy(u_char *, u_char *);
 extern void insertparam();
 extern void makeparam(labelptr);
+extern void initinst(objectptr, objectptr, short);
 extern void searchinst(objectptr, objectptr, short);
 extern stringpart *searchparam(stringpart *);
 extern void unmakeparam(labelptr, stringpart *);
@@ -474,6 +486,14 @@ extern void curtail(u_char *);
 extern objectptr checklibtop();
 extern void removeinst(objinstptr);
 extern void removeparams(objectptr);
+extern void resolveparams(objinstptr);
+
+/* from python.c: */
+
+#ifdef HAVE_PYTHON
+extern void init_interpreter();
+extern void exit_interpreter();
+#endif
 
 /* from rcfile.c: */
 
@@ -523,6 +543,7 @@ extern int checkschem(objectptr, char *);
 extern int checksym(objectptr, char *);
 extern void changeotherpins(labelptr, stringpart *);
 extern void swapschem(Widget, u_int, caddr_t);
+extern void dobeforeswap(Widget, caddr_t, caddr_t);
 extern void schemdisassoc();
 extern void startschemassoc(Widget, caddr_t, caddr_t);
 extern void schemassoc(objectptr, objectptr);
@@ -563,6 +584,7 @@ extern stringpart *findstringpart(int, int *, stringpart *, objinstptr);
 extern void charprint(char *, stringpart *, int);
 extern char *stringprint(stringpart *, objinstptr);
 extern char *textprint(stringpart *, objinstptr);
+extern int textcomp(stringpart *, char *, objinstptr);
 extern int textncomp(stringpart *, char *, objinstptr);
 extern int stringcomp(stringpart *, stringpart *);
 extern int stringcomprelaxed(stringpart *, stringpart *);
