@@ -15,8 +15,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>	/* for usleep() */
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
    
@@ -25,6 +28,7 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include "Xw/Xw.h"
+#include "Xw/MenuBtn.h"
       
 /*----------------------------------------------------------------------*/
 /* Local includes                                                       */
@@ -49,8 +53,14 @@ static PyObject *xc_pause(PyObject *, PyObject *);
 static PyObject *xc_refresh(PyObject *, PyObject *);
 static PyObject *xc_bind(PyObject *, PyObject *);
 static PyObject *xc_unbind(PyObject *, PyObject *);
-void init_interpreter();
-void exit_interpreter();
+static PyObject *xc_popupprompt(PyObject *, PyObject *);
+static PyObject *xc_filepopup(PyObject *, PyObject *);
+static PyObject *xc_newbutton(PyObject *, PyObject *);
+static PyObject *xc_page(PyObject *, PyObject *);
+static PyObject *xc_getcursor();
+static PyObject *xc_reset();
+
+int string_to_type(const char *);
 
 /*----------------------------------------------------------------------*/
 /* External variable declarations                                       */
@@ -67,6 +77,7 @@ extern Display *dpy;
 extern Window win;
 extern keybinding *keylist;
 extern colorindex *colorlist;
+extern ApplicationData appdata;
 
 /*----------------------------------------------------------------------*/
 
@@ -85,6 +96,58 @@ short flags;
 /* fundamental processing here.					*/
 /*--------------------------------------------------------------*/
 
+/*--------------------------------------------------------------*/
+/* Extract text from a popup and call the indicated python	*/
+/* function.							*/
+/*--------------------------------------------------------------*/
+
+void pypromptvalue(Widget w, char *functionptr)
+{
+   sprintf(_STR, "%s('%s')\n", functionptr, _STR2);
+   PyRun_SimpleString(_STR);
+   refresh(NULL, NULL, NULL);
+}
+
+/*--------------------------------------------------------------*/
+/* Pop up a prompt, and return the value to the indicated	*/
+/* python function.						*/
+/*--------------------------------------------------------------*/
+
+static PyObject *pypopupprompt(PyObject *self, PyObject *args, Boolean dofile)
+{
+   const char *pfunction = NULL;
+   const char *prompt = NULL;
+   buttonsave *savebutton = (buttonsave *)malloc(sizeof(buttonsave));
+
+   if (!PyArg_ParseTuple(args, "ss:popupprompt", &prompt, &pfunction))
+      return PyInt_FromLong(-1L);
+
+   _STR2[0] = '\0';
+
+   savebutton->button = NULL;
+   savebutton->buttoncall = NULL;
+   savebutton->dataptr = (char *)pfunction;
+
+   popupprompt(areastruct.area, (char *)prompt, _STR2,
+		pypromptvalue, savebutton, dofile);
+
+   return PyInt_FromLong(0L);
+}
+
+/*--------------------------------------------------------------*/
+/* Wrappers for pypopuprompt()					*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_popupprompt(PyObject *self, PyObject *args)
+{
+   return pypopupprompt(self, args, False);
+}
+
+static PyObject *xc_filepopup(PyObject *self, PyObject *args)
+{
+   return pypopupprompt(self, args, True);
+}
+
 /*--------------------------------------------------------------*/
 /* Create objects and return a handle to that object in the	*/
 /* form of a Python integer Object.				*/
@@ -92,37 +155,50 @@ short flags;
 
 static PyObject *xc_new(PyObject *self, PyObject *args)
 {
-   genericptr *newgen;
    const char *etype;
    int type;
 
+   genericptr *newgen;
+   arcptr *newarc;
+   splineptr *newspline;
+   polyptr *newpoly;
+   labelptr *newlabel;
+   pathptr *newpath;
+   objinstptr *newobjinst;
+
    if (!PyArg_ParseTuple(args, "s:newelement", &etype))
       return NULL;
    
    switch(type = string_to_type(etype)) {
       case ARC:
-         NEW_ARC((arcptr *)newgen, objectdata);
-	 arcdefaults(TOARC(newgen), 0, 0);
+         NEW_ARC(newarc, objectdata);
+	 arcdefaults(*newarc, 0, 0);
+	 newgen = (genericptr *)newarc;
 	 break;
       case SPLINE:
-         NEW_SPLINE((splineptr *)newgen, objectdata);
-	 splinedefaults(TOSPLINE(newgen), 0, 0);
+         NEW_SPLINE(newspline, objectdata);
+	 splinedefaults(*newspline, 0, 0);
+	 newgen = (genericptr *)newspline;
 	 break;
       case POLYGON:
-         NEW_POLY((polyptr *)newgen, objectdata);
-	 polydefaults(TOPOLY(newgen), 4, 0, 0);
+         NEW_POLY(newpoly, objectdata);
+	 polydefaults(*newpoly, 4, 0, 0);
+	 newgen = (genericptr *)newpoly;
 	 break;
       case LABEL:
-         NEW_LABEL((labelptr *)newgen, objectdata);
-	 labeldefaults(TOLABEL(newgen), 0, 0, 0);
+         NEW_LABEL(newlabel, objectdata);
+	 labeldefaults(*newlabel, 0, 0, 0);
+	 newgen = (genericptr *)newlabel;
 	 break;
       case PATH:
-         NEW_PATH((pathptr *)newgen, objectdata);
-	 pathdefaults(TOPATH(newgen), 0, 0);
+         NEW_PATH(newpath, objectdata);
+	 pathdefaults(*newpath, 0, 0);
+	 newgen = (genericptr *)newpath;
 	 break;
       case OBJECT:
-         NEW_OBJINST((objinstptr *)newgen, objectdata);
-	 objectdefaults(TOOBJINST(newgen), NULL, 0, 0);
+         NEW_OBJINST(newobjinst, objectdata);
+	 objectdefaults(*newobjinst, NULL, 0, 0);
+	 newgen = (genericptr *)newobjinst;
 	 break;
       default:
          PyErr_SetString(PyExc_TypeError,
@@ -192,6 +268,225 @@ int string_to_type(const char *etype)
       return -1;
 }
 
+/*--------------------------------------------------------------*/
+/* Convert a Python list to a stringpart* 			*/
+/*--------------------------------------------------------------*/
+
+stringpart *PySetStringParts(PyObject *dval)
+{
+   PyObject *lstr, *litem, *ditem, *sitem, *titem;
+   int i, j, llen;
+   stringpart *strptr, *newpart;
+   char *string;
+   
+   /* If we pass a string, create a list of size 1 to hold the string.	*/
+   /* Otherwise, the argument must be a list.				*/
+
+   if (PyList_Check(dval)) {
+      lstr = dval;
+   }
+   else {
+      if (PyString_Check(dval)) {
+         lstr = PyList_New(1);
+	 PyList_SetItem(lstr, 0, dval);
+      }
+      else {
+         PyErr_SetString(PyExc_TypeError, "argument must be a string or a list");
+	 return NULL;
+      }
+   }
+
+   strptr = NULL;
+   llen = PyList_Size(lstr);
+   for (i = 0; i < llen; i++) {
+      newpart = makesegment(&strptr, NULL);
+      newpart->nextpart = NULL;
+
+      litem = PyList_GetItem(lstr, i);
+      if (PyDict_Check(litem)) {
+	 if ((ditem = PyDict_GetItemString(litem, "Parameter")) != NULL) {
+	    newpart->type = PARAM_START;
+	    newpart->data.paramno = (int)PyInt_AsLong(ditem);
+	 }
+	 else if ((ditem = PyDict_GetItemString(litem, "Font")) != NULL) {
+	    newpart->type = FONT_NAME;
+	    string = PyString_AsString(ditem);
+	    for (j = 0; j < fontcount; j++)
+	       if (!strcmp(fonts[j].psname, string)) break;
+	    if (j == fontcount) loadfontfile(string);
+	    newpart->data.font = j;
+	 }
+	 else if ((ditem = PyDict_GetItemString(litem, "Kern")) != NULL) {
+	    newpart->type = KERN;
+	    newpart->data.kern[0] = (int)PyInt_AsLong(PyTuple_GetItem(ditem, 0));
+	    newpart->data.kern[1] = (int)PyInt_AsLong(PyTuple_GetItem(ditem, 1));
+	 }
+	 else if ((ditem = PyDict_GetItemString(litem, "Color")) != NULL) {
+	    newpart->type = FONT_COLOR;
+	    newpart->data.color = (int)PyInt_AsLong(ditem);
+	 }
+	 else if ((ditem = PyDict_GetItemString(litem, "Font Scale")) != NULL) {
+	    newpart->type = FONT_SCALE;
+	    newpart->data.scale = (float)PyFloat_AsDouble(ditem);
+	 }
+	 else if ((ditem = PyDict_GetItemString(litem, "Text")) != NULL) {
+	    newpart->type = TEXT_STRING;
+	    newpart->data.string = strdup(PyString_AsString(ditem));
+	 }
+      }
+      else if (PyString_Check(litem)) {
+	 string = PyString_AsString(litem);
+	 if (!strcmp(string, "End Parameter")) {
+	    newpart->type = PARAM_END;
+	 }
+	 else if (!strcmp(string, "Tab Stop")) {
+	    newpart->type = TABSTOP;
+	 }
+	 else if (!strcmp(string, "Tab Forward")) {
+	    newpart->type = TABFORWARD;
+	 }
+	 else if (!strcmp(string, "Tab Backward")) {
+	    newpart->type = TABBACKWARD;
+	 }
+	 else if (!strcmp(string, "Return")) {
+	    newpart->type = RETURN;
+	 }
+	 else if (!strcmp(string, "Subscript")) {
+	    newpart->type = SUBSCRIPT;
+	 }
+	 else if (!strcmp(string, "Superscript")) {
+	    newpart->type = SUPERSCRIPT;
+	 }
+	 else if (!strcmp(string, "Normalscript")) {
+	    newpart->type = NORMALSCRIPT;
+	 }
+	 else if (!strcmp(string, "Underline")) {
+	    newpart->type = UNDERLINE;
+	 }
+	 else if (!strcmp(string, "Overline")) {
+	    newpart->type = OVERLINE;
+	 }
+	 else if (!strcmp(string, "No Line")) {
+	    newpart->type = NOLINE;
+	 }
+	 else if (!strcmp(string, "Half Space")) {
+	    newpart->type = HALFSPACE;
+	 }
+	 else if (!strcmp(string, "Quarter Space")) {
+	    newpart->type = QTRSPACE;
+	 }
+	 else {
+	    newpart->type = TEXT_STRING;
+	    newpart->data.string = strdup(string);
+	 }
+      }
+   }
+   return strptr;
+}
+
+/*--------------------------------------------------------------*/
+/* Convert a stringpart* to a Python list 			*/
+/*--------------------------------------------------------------*/
+
+static PyObject *PyGetStringParts(stringpart *thisstring)
+{
+   PyObject *lstr, *sdict, *stup;
+   int i, llen;
+   stringpart *strptr;
+   
+   llen = stringparts(thisstring);
+   lstr = PyList_New(llen);
+   for (strptr = thisstring, i = 0; strptr != NULL;
+      strptr = strptr->nextpart, i++) {
+      switch(strptr->type) {
+	 case TEXT_STRING:
+	    sdict = PyDict_New();
+	    PyDict_SetItem(sdict, PyString_FromString("Text"),
+		  PyString_FromString(strptr->data.string));
+	    PyList_SetItem(lstr, i, sdict);
+	    break;
+	 case PARAM_START:
+	    sdict = PyDict_New();
+	    PyDict_SetItem(sdict, PyString_FromString("Parameter"),
+		  PyInt_FromLong((long)strptr->data.paramno));
+	    PyList_SetItem(lstr, i, sdict);
+	    break;
+	 case PARAM_END:
+	    PyList_SetItem(lstr, i, PyString_FromString("End Parameter"));
+	    break;
+	 case FONT_NAME:
+	    sdict = PyDict_New();
+	    PyDict_SetItem(sdict, PyString_FromString("Font"),
+		  PyString_FromString(fonts[strptr->data.font].psname));
+	    PyList_SetItem(lstr, i, sdict);
+	    break;
+	 case FONT_SCALE:
+	    sdict = PyDict_New();
+	    PyDict_SetItem(sdict, PyString_FromString("Font Scale"),
+		  PyFloat_FromDouble((double)strptr->data.scale));
+	    PyList_SetItem(lstr, i, sdict);
+	    break;
+	 case KERN:
+	    sdict = PyDict_New();
+	    stup = PyTuple_New(2);
+	    PyTuple_SetItem(stup, 0, PyInt_FromLong((long)strptr->data.kern[0]));
+	    PyTuple_SetItem(stup, 1, PyInt_FromLong((long)strptr->data.kern[1]));
+	    PyDict_SetItem(sdict, PyString_FromString("Kern"), stup);
+	    PyList_SetItem(lstr, i, sdict);
+	    break;
+	 case FONT_COLOR:
+	    sdict = PyDict_New();
+	    stup = PyTuple_New(3);
+	    PyTuple_SetItem(stup, 0, PyInt_FromLong(
+		(long)colorlist[strptr->data.color].color.red));
+	    PyTuple_SetItem(stup, 1, PyInt_FromLong(
+		(long)colorlist[strptr->data.color].color.green));
+	    PyTuple_SetItem(stup, 2, PyInt_FromLong(
+		(long)colorlist[strptr->data.color].color.blue));
+	    PyDict_SetItem(sdict, PyString_FromString("Color"), stup); 
+	    PyList_SetItem(lstr, i, sdict);
+	    break;
+	 case TABSTOP:
+	    PyList_SetItem(lstr, i, PyString_FromString("Tab Stop"));
+	    break;
+	 case TABFORWARD:
+	    PyList_SetItem(lstr, i, PyString_FromString("Tab Forward"));
+	    break;
+	 case TABBACKWARD:
+	    PyList_SetItem(lstr, i, PyString_FromString("Tab Backward"));
+	    break;
+	 case RETURN:
+	    PyList_SetItem(lstr, i, PyString_FromString("Return"));
+	    break;
+	 case SUBSCRIPT:
+	    PyList_SetItem(lstr, i, PyString_FromString("Subscript"));
+	    break;
+	 case SUPERSCRIPT:
+	    PyList_SetItem(lstr, i, PyString_FromString("Superscript"));
+	    break;
+	 case NORMALSCRIPT:
+	    PyList_SetItem(lstr, i, PyString_FromString("Normalscript"));
+	    break;
+	 case UNDERLINE:
+	    PyList_SetItem(lstr, i, PyString_FromString("Underline"));
+	    break;
+	 case OVERLINE:
+	    PyList_SetItem(lstr, i, PyString_FromString("Overline"));
+	    break;
+	 case NOLINE:
+	    PyList_SetItem(lstr, i, PyString_FromString("No Line"));
+	    break;
+	 case HALFSPACE:
+	    PyList_SetItem(lstr, i, PyString_FromString("Half Space"));
+	    break;
+	 case QTRSPACE:
+	    PyList_SetItem(lstr, i, PyString_FromString("Quarter Space"));
+	    break;
+      }
+   }
+   return lstr;
+}
+
 /*--------------------------------------------------------------*/
 /* Check if the handle (integer) is an existing element		*/
 /*--------------------------------------------------------------*/
@@ -199,12 +494,36 @@ int string_to_type(const char *etype)
 genericptr *CheckHandle(PyObject *ehandle)
 {
    genericptr *gelem;
-   int eaddr;
+   int i, j, eaddr;
+   objectptr thisobj;
+   Library *thislib;
 
    eaddr = (int)PyInt_AsLong(ehandle);
    for (gelem = objectdata->plist; gelem < objectdata->plist +
 	objectdata->parts; gelem++)
       if ((int)(*gelem) == eaddr) goto exists;
+
+   /* Okay, it isn't in objectdata.  Try all other pages. */
+
+   for (i = 0; i < xobjs.pages; i++) {
+      if (!(thisobj = xobjs.pagelist[i]->pageobj)) continue;
+      for (gelem = thisobj->plist; gelem < thisobj->plist + thisobj->parts; gelem++)
+         if ((int)(*gelem) == eaddr) goto exists;
+   }
+
+   /* Still not found?  Maybe in a library */
+
+   for (i = 0; i < xobjs.numlibs; i++) {
+      thislib = xobjs.userlibs + i;
+      for (j = 0; j < thislib->number; j++) {
+         thisobj = thislib->library[j];
+         for (gelem = thisobj->plist; gelem < thisobj->plist + thisobj->parts; gelem++)
+            if ((int)(*gelem) == eaddr) goto exists;
+      }
+   }
+
+   /* Either in the delete list (where we don't want to go) or	*/
+   /* is an invalid number.					*/
    return NULL;
 
 exists:
@@ -217,7 +536,6 @@ exists:
 
 objectptr CheckPageHandle(PyObject *ehandle)
 {
-   genericptr *gelem;
    int eaddr, pageno;
 
    eaddr = (int)PyInt_AsLong(ehandle);
@@ -226,39 +544,241 @@ objectptr CheckPageHandle(PyObject *ehandle)
    return (objectptr)eaddr;
 }
 
+/*--------------------------------------------------------------*/
+/* Form a 2-tuple from a pair of integers.			*/
+/*--------------------------------------------------------------*/
+
+static PyObject *make_pair(XPoint *thispoint)
+{
+   PyObject *dtup;
+
+   dtup = PyTuple_New(2);
+
+   PyTuple_SetItem(dtup, 0,
+		PyInt_FromLong((long)(thispoint->x)));
+   PyTuple_SetItem(dtup, 1,
+		PyInt_FromLong((long)(thispoint->y)));
+   return dtup;
+}
+
 /*--------------------------------------------------------------*/
 /* Get the properties of a page (returned as a dictionary)	*/
 /*--------------------------------------------------------------*/
 
 static PyObject *xc_getpage(PyObject *self, PyObject *args)
 {
-   objectptr pageobj;
-   PyObject *rdict, *dtup, *dlist;
-   int pageno = -1, i, eaddr;
+   Pagedata *thispage;
+   PyObject *rdict, *dlist;
+   int pageno = areastruct.page + 1, i;
 
    if (!PyArg_ParseTuple(args, "|d:getpage", &pageno))
       return NULL;
 
-   if (pageno < 0) pageno = areastruct.page;
+   if (pageno <= 0 || pageno > xobjs.pages) return NULL;
    else pageno--;
-   if (pageno >= xobjs.pages) return NULL;
-   pageobj = xobjs.pagelist[pageno]->pageobj;
 
-   /* return all the elements properties as a dictionary	*/
+   thispage = xobjs.pagelist[pageno];
+
+   rdict = PyDict_New();
+
+   PyDict_SetItem(rdict, PyString_FromString("filename"),
+	PyString_FromString(thispage->filename));
+
+   PyDict_SetItem(rdict, PyString_FromString("page label"),
+	PyString_FromString(thispage->pageobj->name));
+
+   PyDict_SetItem(rdict, PyString_FromString("output scale"),
+	PyFloat_FromDouble((double)thispage->outscale));
+
+   /* To be done:  change these from internal units to "natural" units */
+
+   PyDict_SetItem(rdict, PyString_FromString("grid space"),
+	PyFloat_FromDouble((double)thispage->gridspace));
+
+   PyDict_SetItem(rdict, PyString_FromString("snap space"),
+	PyFloat_FromDouble((double)thispage->snapspace));
+
+   PyDict_SetItem(rdict, PyString_FromString("orientation"),
+	PyInt_FromLong((long)thispage->orient));
+
+   PyDict_SetItem(rdict, PyString_FromString("output mode"),
+	PyInt_FromLong((long)thispage->pmode));
+
+   PyDict_SetItem(rdict, PyString_FromString("coordinate style"),
+	PyInt_FromLong((long)thispage->coordstyle));
+
+   PyDict_SetItem(rdict, PyString_FromString("page size"),
+	make_pair(&(thispage->pagesize)));
+
+   PyDict_SetItem(rdict, PyString_FromString("drawing scale"),
+	make_pair(&(thispage->drawingscale)));
+
+   return rdict;
+}
+
+/*--------------------------------------------------------------*/
+/* Get the properties of a library (returned as a dictionary)	*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_getlibrary(PyObject *self, PyObject *args)
+{
+   PyObject *rdict, *dlist;
+   const char *lname = NULL;
+   objectptr *curlib = NULL, thisobj;
+   char *errptr;
+   int i, lpage;
+   int curobjs;
+
+   if (!PyArg_ParseTuple(args, "s:getlibrary", &lname))
+      return NULL;
+
+   lpage = strtol(lname, &errptr, 10);
+   if (*lname != '\0' && *errptr == '\0' && lpage <= xobjs.numlibs &&
+		lpage > 0) {	/* numerical */
+      curlib = xobjs.userlibs[lpage - 1].library;
+   }
+   else {
+      for (lpage = 0; lpage < xobjs.numlibs; lpage++) {
+	 if (!strcmp(lname, xobjs.libtop[lpage + LIBRARY]->name)) {
+	    curlib = xobjs.userlibs[lpage - 1].library;
+	    break;
+	 }
+	 else if (!strncmp(xobjs.libtop[lpage + LIBRARY]->name, "Library: ", 9)) {
+	    if (!strcmp(lname, xobjs.libtop[lpage + LIBRARY]->name + 9)) {
+	       curlib = xobjs.userlibs[lpage - 1].library;
+	       break;
+	    }
+	 }
+      }
+   }
+ 
+   if (curlib == NULL) {
+      PyErr_SetString(PyExc_TypeError,
+		"getlibrary() 2nd argument must be a library page or name");
+      return NULL;
+   }
+   curobjs = xobjs.userlibs[lpage - 1].number;
+
+   /* Argument to getlibrary() can be a library name or library page # */
 
    rdict = PyDict_New();
 
    PyDict_SetItem(rdict, PyString_FromString("name"),
-	PyString_FromString(pageobj->name));
+	PyString_FromString(xobjs.libtop[lpage + LIBRARY]->name));
 
-   dlist = PyList_New(pageobj->parts);
-   for (i = 0; i < pageobj->parts; i++)
-      PyList_SetItem(dlist, i, PyInt_FromLong((long)(*(pageobj->plist + i))));
+   if (curobjs > 0) {
+      dlist = PyList_New(curobjs);
+      for (i = 0; i < curobjs; i++) {
+	 thisobj = *(curlib + i);
+         PyList_SetItem(dlist, i, PyString_FromString(thisobj->name));
+      }
+      PyDict_SetItem(rdict, PyString_FromString("objects"), dlist);
+   }
+   return rdict;
+}
+
+/*--------------------------------------------------------------*/
+/* Get the properties of an object (returned as a dictionary)	*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_getobject(PyObject *self, PyObject *args)
+{
+   objectptr thisobj = NULL, *libobj;
+   PyObject *rdict, *dlist;
+   const char *oname;
+   int i, j, k;
+
+   if (!PyArg_ParseTuple(args, "s:getobject", &oname))
+      return NULL;
+
+   for (k = 0; k < xobjs.numlibs; k++) { 
+      for (j = 0; j < xobjs.userlibs[k].number; j++) {
+	 libobj = xobjs.userlibs[k].library + j;
+	 if (!strcmp(oname, (*libobj)->name)) {
+	    thisobj = *libobj;
+	    break;
+	 }
+      }
+      if (thisobj != NULL) break;
+   }
+
+   if (thisobj==NULL) return NULL;
+
+   /* return all the object's properties as a dictionary */
+
+   rdict = PyDict_New();
+
+   PyDict_SetItem(rdict, PyString_FromString("name"),
+	PyString_FromString(thisobj->name));
+   PyDict_SetItem(rdict, PyString_FromString("width"), 
+	PyInt_FromLong((long)(thisobj->width)));
+   PyDict_SetItem(rdict, PyString_FromString("height"), 
+	PyInt_FromLong((long)(thisobj->height)));
+
+   dlist = PyList_New(thisobj->parts);
+   for (i = 0; i < thisobj->parts; i++)
+      PyList_SetItem(dlist, i, PyInt_FromLong((long)(*(thisobj->plist + i))));
    PyDict_SetItem(rdict, PyString_FromString("parts"), dlist);
 
+   if (thisobj->num_params > 0) {
+      dlist = PyList_New(thisobj->num_params);
+      for (i = 0; i < thisobj->num_params; i++) {
+	 if (thisobj->params[i]->type == XC_SHORT)
+            PyList_SetItem(dlist, i,
+		PyInt_FromLong((long)(thisobj->params[i]->parameter.svalue)));
+	 else if (thisobj->params[i]->type == XC_STRING)
+            PyList_SetItem(dlist, i,
+		PyGetStringParts(thisobj->params[i]->parameter.string));
+      }
+      PyDict_SetItem(rdict, PyString_FromString("parameters"), dlist);
+   }
    return rdict;
 }
 
+/*--------------------------------------------------------------*/
+/* Reset the current page					*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_reset()
+{
+   resetbutton(NULL, NULL, NULL);
+   return PyInt_FromLong(0L);
+}
+
+/*--------------------------------------------------------------*/
+/* Go to the specified page					*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_page(PyObject *self, PyObject *args)
+{
+   int pageno;
+
+   if (!PyArg_ParseTuple(args, "i:page", &pageno))
+      return NULL;
+
+   newpage(pageno - 1);
+   return PyInt_FromLong((long)pageno);
+}
+
+/*--------------------------------------------------------------*/
+/* Get the cursor position					*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_getcursor()
+{
+   PyObject *cursorpos;
+   XPoint newpos;
+
+   newpos = UGetCursorPos();
+   u2u_snap(&newpos);
+   
+   cursorpos = PyTuple_New(2);
+   PyTuple_SetItem(cursorpos, 0, PyInt_FromLong((long)newpos.x));
+   PyTuple_SetItem(cursorpos, 1, PyInt_FromLong((long)newpos.y));
+
+   return cursorpos;
+}
+
 /*--------------------------------------------------------------*/
 /* Get the properties of an element (returned as a dictionary)	*/
 /*--------------------------------------------------------------*/
@@ -266,8 +786,8 @@ static PyObject *xc_getpage(PyObject *self, PyObject *args)
 static PyObject *xc_getattr(PyObject *self, PyObject *args)
 {
    genericptr *gelem;
-   PyObject *ehandle, *rdict, *dtup, *dlist, *lstr, *sdict, *stup;
-   int i, eaddr, llen;
+   PyObject *ehandle, *rdict, *dlist, *lstr, *sdict, *stup;
+   int i, llen;
    char *tstr;
    stringpart *strptr;
    
@@ -299,12 +819,8 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 
    switch((*gelem)->type) {
       case LABEL:
-	 dtup = PyTuple_New(2);
-	 PyTuple_SetItem(dtup, 0,
-		PyInt_FromLong((long)TOLABEL(gelem)->position.x));
-	 PyTuple_SetItem(dtup, 1,
-		PyInt_FromLong((long)TOLABEL(gelem)->position.y));
-	 PyDict_SetItem(rdict, PyString_FromString("position"), dtup);
+	 PyDict_SetItem(rdict, PyString_FromString("position"), 
+		make_pair(&(TOLABEL(gelem)->position)));
 	 PyDict_SetItem(rdict, PyString_FromString("rotation"),
 		PyInt_FromLong((long)TOLABEL(gelem)->rotation));
 	 PyDict_SetItem(rdict, PyString_FromString("scale"),
@@ -315,84 +831,8 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 	 PyDict_SetItem(rdict, PyString_FromString("pin"),
 		PyInt_FromLong((long)TOLABEL(gelem)->pin));
 #endif
-	 llen = stringparts(TOLABEL(gelem)->string);
-	 lstr = PyList_New(llen);
-	 for (strptr = TOLABEL(gelem)->string, i = 0; strptr != NULL;
-		strptr = strptr->nextpart, i++) {
-	    switch(strptr->type) {
-	       case TEXT_STRING:
-		  PyList_SetItem(lstr, i, PyString_FromString(strptr->data.string));
-		  break;
-	       case PARAM_START:
-		  sdict = PyDict_New();
-		  PyDict_SetItem(sdict, PyString_FromString("Parameter"),
-		  	PyInt_FromLong((long)strptr->data.paramno));
-		  PyList_SetItem(lstr, i, sdict);
-		  break;
-	       case FONT_NAME:
-		  sdict = PyDict_New();
-		  PyDict_SetItem(sdict, PyString_FromString("Font"),
-		  	PyString_FromString(fonts[strptr->data.paramno].psname));
-		  PyList_SetItem(lstr, i, sdict);
-		  break;
-	       case KERN:
-		  sdict = PyDict_New();
-		  stup = PyTuple_New(2);
-		  PyTuple_SetItem(stup, 0, PyInt_FromLong((long)strptr->data.kern[0]));
-		  PyTuple_SetItem(stup, 1, PyInt_FromLong((long)strptr->data.kern[1]));
-		  PyDict_SetItem(sdict, PyString_FromString("Kern"), stup);
-		  PyList_SetItem(lstr, i, sdict);
-		  break;
-	       case FONT_COLOR:
-		  sdict = PyDict_New();
-		  stup = PyTuple_New(3);
-		  PyTuple_SetItem(stup, 0, PyInt_FromLong(
-			(long)colorlist[strptr->data.color].color.red));
-		  PyTuple_SetItem(stup, 1, PyInt_FromLong(
-			(long)colorlist[strptr->data.color].color.green));
-		  PyTuple_SetItem(stup, 2, PyInt_FromLong(
-			(long)colorlist[strptr->data.color].color.blue));
-		  PyDict_SetItem(sdict, PyString_FromString("Color"), stup); 
-		  PyList_SetItem(lstr, i, sdict);
-		  break;
-	       case TABSTOP:
-		  PyList_SetItem(lstr, i, PyString_FromString("Tab Stop"));
-		  break;
-	       case TABFORWARD:
-		  PyList_SetItem(lstr, i, PyString_FromString("Tab Forward"));
-		  break;
-	       case TABBACKWARD:
-		  PyList_SetItem(lstr, i, PyString_FromString("Tab Backward"));
-		  break;
-	       case RETURN:
-		  PyList_SetItem(lstr, i, PyString_FromString("Return"));
-		  break;
-	       case SUBSCRIPT:
-		  PyList_SetItem(lstr, i, PyString_FromString("Subscript"));
-		  break;
-	       case SUPERSCRIPT:
-		  PyList_SetItem(lstr, i, PyString_FromString("Superscript"));
-		  break;
-	       case NORMALSCRIPT:
-		  PyList_SetItem(lstr, i, PyString_FromString("Normalscript"));
-		  break;
-	       case UNDERLINE:
-		  PyList_SetItem(lstr, i, PyString_FromString("Underline"));
-		  break;
-	       case OVERLINE:
-		  PyList_SetItem(lstr, i, PyString_FromString("Overline"));
-		  break;
-	       case NOLINE:
-		  PyList_SetItem(lstr, i, PyString_FromString("No Line"));
-		  break;
-	       case HALFSPACE:
-		  PyList_SetItem(lstr, i, PyString_FromString("Half Space"));
-		  break;
-	       case QTRSPACE:
-		  PyList_SetItem(lstr, i, PyString_FromString("Quarter Space"));
-		  break;
-	    }
-	 }
+	 lstr = PyGetStringParts(TOLABEL(gelem)->string);
+
 	 PyDict_SetItem(rdict, PyString_FromString("string"), lstr);
 	 break;
       case POLYGON:
@@ -402,12 +842,7 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 		PyFloat_FromDouble((double)TOPOLY(gelem)->width));
 	 dlist = PyList_New(TOPOLY(gelem)->number);
 	 for (i = 0; i < TOPOLY(gelem)->number; i++) {
-	    dtup = PyTuple_New(2);
-	    PyTuple_SetItem(dtup, 0,
-		   PyInt_FromLong((long)TOPOLY(gelem)->points[i].x));
-	    PyTuple_SetItem(dtup, 1,
-		   PyInt_FromLong((long)TOPOLY(gelem)->points[i].y));
-	    PyList_SetItem(dlist, i, dtup);
+	    PyList_SetItem(dlist, i, make_pair(&(TOPOLY(gelem)->points[i])));
 	 }
 	 PyDict_SetItem(rdict, PyString_FromString("points"), dlist);
 	 break;
@@ -424,12 +859,8 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 		PyFloat_FromDouble((double)TOARC(gelem)->angle1));
 	 PyDict_SetItem(rdict, PyString_FromString("end angle"),
 		PyFloat_FromDouble((double)TOARC(gelem)->angle2));
-	 dtup = PyTuple_New(2);
-	 PyTuple_SetItem(dtup, 0,
-		PyInt_FromLong((long)TOARC(gelem)->position.x));
-	 PyTuple_SetItem(dtup, 1,
-		PyInt_FromLong((long)TOARC(gelem)->position.y));
-	 PyDict_SetItem(rdict, PyString_FromString("position"), dtup);
+	 PyDict_SetItem(rdict, PyString_FromString("position"),
+		make_pair(&(TOARC(gelem)->position)));
 	 break;
       case SPLINE:
 	 PyDict_SetItem(rdict, PyString_FromString("style"), 
@@ -438,12 +869,7 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 		PyFloat_FromDouble((double)TOSPLINE(gelem)->width));
 	 dlist = PyList_New(4);
 	 for (i = 0; i < 4; i++) {
-	    dtup = PyTuple_New(2);
-	    PyTuple_SetItem(dtup, 0,
-		   PyInt_FromLong((long)TOSPLINE(gelem)->ctrl[i].x));
-	    PyTuple_SetItem(dtup, 1,
-		   PyInt_FromLong((long)TOSPLINE(gelem)->ctrl[i].y));
-	    PyList_SetItem(dlist, i, dtup);
+	    PyList_SetItem(dlist, i, make_pair(&(TOSPLINE(gelem)->ctrl[i])));
 	 }
 	 PyDict_SetItem(rdict, PyString_FromString("control points"), dlist);
 	 break;
@@ -460,11 +886,8 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 	 PyDict_SetItem(rdict, PyString_FromString("parts"), dlist);
 	 break;
       case OBJECT:
-	 dtup = PyTuple_New(2);
-	 PyTuple_SetItem(dtup, 0,
-		PyInt_FromLong((long)TOOBJINST(gelem)->position.x));
-	 PyTuple_SetItem(dtup, 1,
-		PyInt_FromLong((long)TOOBJINST(gelem)->position.y));
+	 PyDict_SetItem(rdict, PyString_FromString("position"), 
+		make_pair(&(TOOBJINST(gelem)->position)));
 	 PyDict_SetItem(rdict, PyString_FromString("rotation"),
 		PyInt_FromLong((long)TOLABEL(gelem)->rotation));
 	 PyDict_SetItem(rdict, PyString_FromString("scale"),
@@ -482,7 +905,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 {
    genericptr *gelem;
    PyObject *ehandle, *attrdict, *dval, *pval, *qval;
-   int i, eaddr;
+   int i;
 
    if (!PyArg_ParseTuple(args, "OO:setattr", &ehandle, &attrdict))
       return NULL;
@@ -642,7 +1065,8 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	    TOLABEL(gelem)->justify = PyInt_AsLong(dval);
 	 }
 	 if ((dval = PyDict_GetItemString(attrdict, "string")) != NULL) {
-	    /* To be dealt with. . . requires a list of string parts */
+	    freelabel(TOLABEL(gelem)->string);
+	    TOLABEL(gelem)->string = PySetStringParts(dval);
 	 }
 #ifdef SCHEMA
 	 if ((dval = PyDict_GetItemString(attrdict, "pin")) != NULL) {
@@ -722,6 +1146,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	 break;
    }
 
+   calcbbox(objectdata);
    return PyInt_FromLong((long)(*gelem));
 }
 /*--------------------------------------------------------------*/
@@ -863,7 +1288,7 @@ static PyObject *xc_library(PyObject *self, PyObject *args)
       flags |= LIBLOADED;   /* Pass through a Python variable? */
    }
 
-   if (libnum >= xobjs.numlibs)
+   if (libnum >= xobjs.numlibs || libnum < 0)
       libnum = createlibrary();
    else
       libnum += LIBRARY - 1;
@@ -910,6 +1335,7 @@ static PyObject *xc_bind(PyObject *self, PyObject *args)
    const char *keyname = NULL;
    const char *function = NULL;
    PyObject *retobj;
+   int keywstate;
 
    if (!PyArg_ParseTuple(args, "|ss:bind", &keyname, &function))
       return NULL;
@@ -940,7 +1366,7 @@ static PyObject *xc_bind(PyObject *self, PyObject *args)
    /* One argument?  Argument is a key or a function */
    else if (!function) {
       char *binding;
-      int func = -1, keywstate;
+      int func = -1;
       keywstate = string_to_key(keyname);
       if (keywstate == 0) { /* first argument (keyname) is function? */
 	 keywstate = -1;
@@ -951,7 +1377,12 @@ static PyObject *xc_bind(PyObject *self, PyObject *args)
       free(binding);
    }
    else {
-      add_keybinding(keyname, function);
+      if (add_keybinding(keyname, function) < 0) {
+	 /* Function may be a Python function, not a C function */
+         keywstate = string_to_key(keyname);
+	 sprintf(_STR2, "keydict[%d] = %s\n", keywstate, function);
+	 PyRun_SimpleString(_STR2);
+      }
       retobj = PyString_FromString(keyname);
    }
    return retobj;
@@ -961,8 +1392,8 @@ static PyObject *xc_bind(PyObject *self, PyObject *args)
 
 static PyObject *xc_unbind(PyObject *self, PyObject *args)
 {
-   const char *keyname;
-   const char *function;
+   const char *keyname = NULL;
+   const char *function = NULL;
 
    if (!PyArg_ParseTuple(args, "ss:unbind", &keyname, &function))
       return NULL;
@@ -1005,6 +1436,72 @@ static PyObject *xc_refresh(PyObject *self, PyObject *args)
    return PyInt_FromLong(0L);
 }
 
+/*--------------------------------------------------------------*/
+/* Callback procedure for Python-generated buttons		*/
+/*--------------------------------------------------------------*/
+
+int pybutton(Widget w, caddr_t clientdata, caddr_t calldata)
+{  
+   int status;
+
+   sprintf(_STR2, "buttondict[%d]()\n", w);
+   status = PyRun_SimpleString(_STR2);
+   refresh(NULL, NULL, NULL);
+   return status;
+}
+
+/*--------------------------------------------------------------*/
+/* Add a new button to the specified (by name) cascade menu	*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_newbutton(PyObject *self, PyObject *args)
+{
+   const char *pname = NULL;
+   const char *newbname = NULL;
+   const char *pfunction = NULL;
+   int status;
+   Arg	wargs[1];
+   int  i, n = 0;
+   Widget newbutton, cascade, tbutton;
+
+   if (!PyArg_ParseTuple(args, "sss:newbutton", &pname, &newbname, &pfunction))
+      return NULL;
+
+   for (i = 0; i < MaxMenuWidgets; i++) {
+      tbutton = menuwidgets[i];
+      cascade = XtParent(tbutton);
+      if (!strcmp(XtName(cascade), pname)) break;
+   }
+   if (i == MaxMenuWidgets) {
+      Wprintf("Cannot find specified menu.");
+      return;
+   }
+
+   XtnSetArg(XtNfont, appdata.xcfont);
+   newbutton = XtCreateWidget(newbname, XwmenubuttonWidgetClass,
+	 cascade, wargs, n);
+
+   XtAddCallback (newbutton, XtNselect, (XtCallbackProc)pybutton, newbutton);
+   XtManageChild(newbutton);
+
+   sprintf(_STR2, "buttondict[%d] = %s\n", newbutton, pfunction);
+   status = PyRun_SimpleString(_STR2);
+   return PyInt_FromLong((long)status);
+}
+
+/*----------------------------------------------------------------*/
+/* Execute a python command from the key-function pair dictionary */
+/*----------------------------------------------------------------*/
+
+int python_key_command(int keystate)
+{
+   int status;
+   sprintf(_STR2, "keydict[%d]()\n", keystate);
+   status = PyRun_SimpleString(_STR2);
+   refresh(NULL, NULL, NULL);
+   return status;
+}
+
 /*--------------------------------------------------------------*/
 /* Declaration of the xcircuit Python functions			*/
 /*--------------------------------------------------------------*/
@@ -1017,12 +1514,20 @@ static PyMethodDef xc_methods[] = {
    {"color",		xc_color,	1},
    {"newelement",	xc_new,		1},
    {"getpage",		xc_getpage,	1},
+   {"getlibrary",	xc_getlibrary,	1},
+   {"getobject",	xc_getobject,	1},
    {"getattr",		xc_getattr,	1},
    {"setattr",		xc_setattr,	1},
    {"refresh",		xc_refresh,	1},
    {"pause",		xc_pause,	1},
    {"bind",		xc_bind,	1},
    {"unbind",		xc_unbind,	1},
+   {"getcursor",	xc_getcursor,	1},
+   {"popupprompt",	xc_popupprompt, 1},
+   {"filepopup",	xc_filepopup,	1},
+   {"newbutton",	xc_newbutton,	1},
+   {"reset",		xc_reset,	1},
+   {"page",		xc_page,	1},
    {NULL,	NULL}			/* sentinel */
 };
 
@@ -1036,6 +1541,8 @@ void init_interpreter()
    PyImport_AddModule("xc");
    Py_InitModule("xc", xc_methods);
    PyRun_SimpleString("from xc import *\n");
+   PyRun_SimpleString("keydict = {}\n");	/* initialize key/function pairs */
+   PyRun_SimpleString("buttondict = {}\n");  /* initialize button/function pairs */
 }
 
 /*--------------------------------------------------------------*/
