@@ -10,10 +10,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
+#include <unistd.h>
 #include <time.h>
+#include <sys/wait.h>   /* for waitpid() */
 #include <sys/types.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -189,7 +193,7 @@ void wait_for_done()
 void start_gs()
 {
    int std_out[2], std_err[2], ret;
-#ifdef SVR4
+#ifdef HAVE_PUTENV
    static char env_str1[128], env_str2[64];
 #endif
 
@@ -227,7 +231,7 @@ void start_gs()
 	 close(std_err[1]);
 #endif
 
-#ifdef SVR4
+#ifdef HAVE_PUTENV
    	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
 	 putenv(env_str1);
 #ifdef DOUBLEBUFFER
@@ -247,7 +251,7 @@ void start_gs()
 	 setenv("GHOSTVIEW", _STR, True);
 #endif
 #endif
-         execl(GS_EXEC, "gs", "-dNOPAUSE -");
+         execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", NULL);
 	 gsproc = -1;
 	 printf("Exec of gs failed\n");
 	 return;
