@@ -10,7 +10,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 
@@ -24,7 +26,6 @@
 /*-------------------------------------------------------------------------*/
 
 #include "xcircuit.h"
-#include "cursors.h"
 #include "colordefs.h"
 #include "menudep.h"
 
@@ -45,6 +46,7 @@ extern Widget     menuwidgets[];
 extern Widget	  netbutton;
 extern objectpair *pushlist;
 extern char	  _STR[150];
+extern char	  _STR2[250];
 
 /*-------------------------------------------------------------------------*/
 #ifdef SCHEMA
@@ -67,6 +69,12 @@ void callgennet(Widget w, u_int mode, caddr_t calldata)
      case 2:
         gennet("pcb", "pcb");
 	break;
+     case 3:
+        gennet("flatspice", "fspc");
+	break;
+     case 4:
+	gennet("idxpcb", "");	/* Auto-numbering:  no output generated */
+	break;
    }
 }
 
@@ -103,9 +111,9 @@ void doxschema(Widget w, caddr_t clientdata, caddr_t calldata)
    drawarea(NULL, NULL, NULL);
 }
 
-/*----------------------------------*/
-/* Find connectivity of a selection */
-/*----------------------------------*/
+/*--------------------------------------*/
+/* Find connectivity of a selection	*/
+/*--------------------------------------*/
 
 void startconnect(Widget button, caddr_t clientdata, caddr_t calldata)
 {
@@ -454,11 +462,10 @@ void findsubschems(int toppage, objectptr cschem)
 	       /* printf("Searching for dependencies of symbol %s\n", cobj->name); */
 	       findsubschems(toppage, cobj->symschem);
 	    }
-/*
-	    else {
-	       printf("Notice:  Symbol on its own schematic\n");
-	    }
-*/
+	 }
+	 else if (cobj->schemtype != FUNDAMENTAL && cobj->schemtype != TRIVIAL) {
+	    /* Check symbols acting as their own schematics */
+	    findsubschems(toppage, cobj);
 	 }
       }
    }
@@ -566,7 +573,8 @@ int checksym(objectptr symobj, char *cname)
 
 /*----------------------------------------------------------------------*/
 /* Find location of corresponding pin in symbol/schematic and change it */
-/* to the text of the indicated label.					*/
+/* to the text of the indicated label.  If "newlabel" is NULL, then	*/
+/* change the other label from pin to normal text.			*/
 /*----------------------------------------------------------------------*/
 
 void changeotherpins(labelptr newlabel, stringpart *oldstring)
@@ -581,8 +589,12 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
       if ((*tgen)->type == LABEL) {
 	 tlab = TOLABEL(tgen);
 	 if (!stringcomp(tlab->string, oldstring)) {
-	    free(tlab->string);
-	    tlab->string = stringcopy(newlabel->string);
+	    if (newlabel != NULL) {
+	       free(tlab->string);
+	       tlab->string = stringcopy(newlabel->string);
+	    }
+	    else
+	       tlab->pin = NORMAL;
 	 }
       }
    }
@@ -633,6 +645,7 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
 	 initmem(*newobject);
 	 (*newobject)->schemtype = SYMBOL;
 	 (*newobject)->hidden = False;
+	 (*newobject)->devname = NULL;
 	 objectdata = *newobject;
 
 	 /* Generate the default bounding box for a size-zero object */
@@ -703,6 +716,48 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
    setsymschem();
 }
 
+/*----------------------------------------------------------------------*/
+/* Wrapper for swapschem() when generating a new symbol.		*/
+/*----------------------------------------------------------------------*/
+
+void makesymbol(Widget w, caddr_t calldata)
+{
+   /* copy name from popup prompt buffer and check */
+
+   strcpy(_STR, _STR2);
+   checkname(objectdata);
+   swapschem(w, (u_int)1, calldata);
+}
+
+/*----------------------------------------------------------------------*/
+/* Check name before doing a swap:  If name begins with "Page", prompt	*/
+/* for the object name, as you would when doing selectsave().		*/ 
+/*----------------------------------------------------------------------*/
+
+void dobeforeswap(Widget w, caddr_t clientdata, caddr_t calldata)
+{
+   buttonsave *popdata = (buttonsave *)malloc(sizeof(buttonsave));
+
+   /* Check if this function is valid */
+
+   if (areastruct.schemon == False) return;
+
+   /* Check for requirement to change the name before creating the symbol */
+
+   if ((objectdata->symschem == NULL) && (objectdata->schemtype == SCHEMATIC)
+		&& (strstr(objectdata->name, "Page ") != NULL)) {
+
+      /* Get a name for the new object */
+
+      eventmode = NORMAL_MODE;
+      popdata->dataptr = NULL;
+      popdata->button = NULL; /* indicates that no button is assc'd w/ the popup */
+      popupprompt(w, "Enter name for new object:", "\0", makesymbol, popdata, False);
+   }
+   else
+      swapschem(w, (u_int)1, calldata);
+}
+
 /*------------------------------------------*/
 /* Disassociate a symbol from its schematic */
 /*------------------------------------------*/
