@@ -10,7 +10,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 #include <math.h>
@@ -24,7 +26,6 @@
 /*-------------------------------------------------------------------------*/
 
 #include "xcircuit.h"
-#include "cursors.h"
 #include "colordefs.h"
 
 /*----------------------------------------------------------------------*/
@@ -322,7 +323,7 @@ short *selectobject(short class, objectptr selobj, objinstptr selinst)
    short	*newselect, *foundtmp, *foundlist;
 
    genericptr	*curgen;
-   XPoint 	bboxpts[4], newboxpts[4];
+   XPoint 	newboxpts[4];
    XEvent 	event;
    XButtonEvent *buttonevent = (XButtonEvent *)(&event);
    Boolean	selected, unselect = False;
@@ -464,6 +465,7 @@ short *selectobject(short class, objectptr selobj, objinstptr selinst)
 	 break;
 
       default: {
+	 newselect = NULL;
 	 XSetFunction(dpy, areastruct.gc, GXcopy);
 	 for (foundtmp = foundlist; foundtmp < foundlist + found; foundtmp++) {
             XSetForeground(dpy, areastruct.gc, QUERYCOLOR);
@@ -896,14 +898,10 @@ void startselect(XtIntervalId id, caddr_t clientdata)
 
 void trackselarea(Widget w, XtPointer clientdata, XMotionEvent *event)
 {
-   Window nullwin;
-   int    nullint, xpos, ypos;
    XPoint newpos;
    u_int  nullui;
 
-   XQueryPointer(dpy, win, &nullwin, &nullwin, &nullint, &nullint, &xpos, &ypos,
-	&nullui);
-   window_to_user(xpos, ypos, &newpos);
+   newpos = UGetCursorPos();
    if (newpos.x == areastruct.save.x && newpos.y == areastruct.save.y) return;
 
    UDrawBox(areastruct.origin, areastruct.save);
