@@ -11,7 +11,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if !defined(__DARWIN__)
+#if defined(DARWIN)
+#include <sys/malloc.h>
+#else
 #include <malloc.h>
 #endif
 #include <signal.h>
@@ -20,6 +22,7 @@
 #include <errno.h>
 #include <limits.h>
 #include <locale.h>
+#include <unistd.h>   /* for unlink() */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -576,11 +579,6 @@ void quit(Widget w, caddr_t clientdata, caddr_t calldata)
 
    exit_gs();
 
-#ifdef HAVE_PYTHON
-   /* exit the Python interpreter, if enabled */
-   exit_interpreter();
-#endif
-
    /* remove any temporary files created for background rendering */
 
    for (i = 0; i < xobjs.pages; i++) {
@@ -605,7 +603,13 @@ void quit(Widget w, caddr_t clientdata, caddr_t calldata)
    }
    free(xobjs.tempfile);
 
+#ifdef HAVE_PYTHON
+   /* exit by exiting the Python interpreter, if enabled */
+   exit_interpreter();
+#else
    exit(0);
+#endif
+
 }
 
 /*--------------*/
@@ -832,7 +836,7 @@ void updatename(Widget button, WidgetList callstruct, caddr_t calldata)
 void popupprompt(Widget button, char *request, char *current, void (*function)(),
 	buttonsave *datastruct, Boolean fileprompt)
 {
-    Arg         wargs[8];
+    Arg         wargs[9];
     Widget      popup, dialog, okbutton, cancelbutton, entertext;
     Widget	staticarea;
     XWMHints	*wmhints;	/* for proper input focus */
@@ -859,7 +863,7 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
     XtnSetArg(XtNx, xpos);
     XtnSetArg(XtNy, ypos);
     popup = XtCreatePopupShell("prompt", transientShellWidgetClass,
-        button, wargs, n); n = 0;
+        button == NULL ? areastruct.area : button, wargs, n); n = 0;
     popups++;
 
     XtnSetArg(XtNlayout, XwIGNORE);
@@ -903,11 +907,13 @@ void popupprompt(Widget button, char *request, char *current, void (*function)()
        XtnSetArg(XtNx, 20);
        XtnSetArg(XtNy, ROWHEIGHT + 10 + (fileprompt ? LISTHEIGHT : 0));
        XtnSetArg(XtNheight, ROWHEIGHT + 5);
+       XtnSetArg(XtNwidth, width - 40);
        XtnSetArg(XtNstring, current);
        pdptr = strchr(current, '.');
        posit = (pdptr != NULL) ? (short)(pdptr - current) : strlen(current);
        XtnSetArg(XtNinsertPosition, posit);
-       XtnSetArg(XtNgrow, XwGrowHorizontal);
+       XtnSetArg(XtNscroll, XwAutoScrollHorizontal);
+       XtnSetArg(XtNwrap, XwWrapOff);
        XtnSetArg(XtNfont, appdata.textfont);
        entertext = XtCreateManagedWidget("Edit", XwtextEditWidgetClass,
 	   dialog, wargs, n); n = 0; 
@@ -956,7 +962,7 @@ propstruct okstruct[MAXPROPS], fpokstruct;
 void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
 {
    buttonsave  *savebutton;
-   Arg         wargs[8];
+   Arg         wargs[9];
    Widget      popup, dialog, okbutton, titlearea, wrbutton;
    Widget      fpentertext, fpokay, autobutton;
    WidgetList  staticarea, entertext, okays;
@@ -976,7 +982,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
    struct stat statbuf;
    static char defaultTranslations[] = "<Key>Return:	execute()";
 
-   if (pushlist != NULL) {
+   if (is_page(objectdata) == -1) {
       Wprintf("Can only save a top-level page!");
       return;
    }
@@ -1022,7 +1028,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
    sprintf(edit[5], "%s", (xobjs.pagelist[areastruct.page]->orient == 0)
 	? "Portrait" : "Landscape");
    sprintf(edit[6], "%s", (xobjs.pagelist[areastruct.page]->pmode & 1)
-	? "Full page (PS)" : "Embedded (EPS)");
+	? "Full page" : "Embedded (EPS)");
    function[0] = setfilename;
    function[1] = setpagelabel;
    function[2] = setfloat;
@@ -1134,7 +1140,8 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
          pdptr = strchr(edit[i], '.');
          posit = (pdptr != NULL) ? (short)(pdptr - edit[i]) : strlen(edit[i]);
          XtnSetArg(XtNinsertPosition, posit);
-         XtnSetArg(XtNgrow, XwGrowHorizontal);
+         XtnSetArg(XtNscroll, XwAutoScrollHorizontal);
+         XtnSetArg(XtNwrap, XwWrapOff);
          XtnSetArg(XtNfont, appdata.textfont);
          entertext[i] = XtCreateManagedWidget("Edit", XwtextEditWidgetClass,
 	     dialog, wargs, n); n = 0; 
@@ -1177,7 +1184,7 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
       Make this text structure always but allow it to be managed and
       unmanaged as necessary. */
 
-   XtnSetArg(XtNx, 260);
+   XtnSetArg(XtNx, 240);
    XtnSetArg(XtNy, ROWHEIGHT + 10 + (10 * ROWHEIGHT));
    XtnSetArg(XtNset, (xobjs.pagelist[areastruct.page]->pmode & 2) ? True : False);
    XtnSetArg(XtNsquare, True);
@@ -1196,16 +1203,17 @@ void dooutput(Widget button, caddr_t clientdata, caddr_t calldata)
 	 (float)xobjs.pagelist[areastruct.page]->pagesize.x / 72.0,
 	 (float)xobjs.pagelist[areastruct.page]->pagesize.y / 72.0);
    }
-   XtnSetArg(XtNx, 260);
+   XtnSetArg(XtNx, 240);
    XtnSetArg(XtNy, ROWHEIGHT + 10 + (12 * ROWHEIGHT));
    XtnSetArg(XtNheight, ROWHEIGHT + 5);
    XtnSetArg(XtNstring, fpedit);
-   XtnSetArg(XtNwidth, width - owidth - 300);
+   XtnSetArg(XtNwidth, width - owidth - 280);
    pdptr = strchr(fpedit, '.');
    posit = (pdptr != NULL) ? (short)(pdptr - fpedit) : strlen(fpedit);
-   XtnSetArg(XtNinsertPosition, posit);
-   XtnSetArg(XtNgrow, XwGrowHorizontal);
+   XtnSetArg(XtNscroll, XwAutoScrollHorizontal);
+   XtnSetArg(XtNwrap, XwWrapOff);
    XtnSetArg(XtNfont, appdata.textfont);
+   XtnSetArg(XtNinsertPosition, posit);
    fpentertext = XtCreateWidget("fpedit", XwtextEditWidgetClass,
        dialog, wargs, n); n = 0;
 
@@ -1495,7 +1503,7 @@ XtConverter CvtStringToPixel(XrmValuePtr args, int *nargs, XrmValuePtr fromVal,
    static XColor cvcolor;
    XColor cvexact;
 
-   if (dpy == NULL) return;
+   if (dpy == NULL) return NULL;
 
    if (*nargs != 0)
       XtWarning("String to Pixel conversion needs no arguments");
@@ -1512,6 +1520,7 @@ XtConverter CvtStringToPixel(XrmValuePtr args, int *nargs, XrmValuePtr fromVal,
 
    toVal->size = sizeof(u_long);
    toVal->addr = (caddr_t) &(cvcolor.pixel);
+   return NULL;
 }
 
 /*-------------------------------------------------------------------------*/
@@ -1697,7 +1706,7 @@ int main(int argc, char *argv[])
    /* setlocale() to work.					  */
    /*-------------------------------------------------------------*/
 
-#ifdef SVR4
+#ifdef HAVE_PUTENV
    putenv("LC_ALL=en_US");
    putenv("LC_NUMERIC=en_US");
    putenv("LANG=POSIX");
@@ -1713,6 +1722,7 @@ int main(int argc, char *argv[])
    /*---------------------------*/
 
 #ifdef SCHEMA
+   areastruct.pinpointon = False;
    areastruct.schemon = False;
    for (k = argc - 1; k > 0; k--) {
       if (!strncmp(argv[k], SCHEM_FLAG, strlen(SCHEM_FLAG))) {
@@ -1816,6 +1826,16 @@ int main(int argc, char *argv[])
 
    XtSetLanguageProc(NULL, NULL, NULL);
 
+   /*-----------------------------------*/
+   /* Set pointer to own XDefaults file */
+   /*-----------------------------------*/
+
+#ifdef HAVE_PUTENV
+   putenv("XAPPLRESDIR=" XAPPLRESDIR);
+#else
+   setenv("XAPPLRESDIR", XAPPLRESDIR, 1);
+#endif
+
    /*-----------------------------*/
    /* Create the widget hierarchy */
    /*-----------------------------*/
@@ -1848,13 +1868,14 @@ int main(int argc, char *argv[])
    XtGetApplicationResources(top, &appdata, resources, XtNumber(resources), 
 	NULL, 0);
 
+   n = 0;
    XtnSetArg(XtNwidth, appdata.width);
    XtnSetArg(XtNheight, appdata.height);
    XtnSetArg(XtNforeground, appdata.fg);
    XtnSetArg(XtNbackground, appdata.bg);
    XtnSetArg(XtNcolormap, cmap);
-
    XtSetValues(top, wargs, n); n = 0;
+
    form = XtCreateManagedWidget("Form", XwformWidgetClass, top, NULL, 0);
 
    /* Set up the buttons and Graphics drawing area */
@@ -2252,14 +2273,18 @@ int main(int argc, char *argv[])
 
    XDefineCursor (dpy, win, CROSS);
 
-   /*-------------------------------------------------*/
-   /* Parse the command line for initial file to load */
-   /*-------------------------------------------------*/
+   /*----------------------------------------------------*/
+   /* Parse the command line for initial file to load.   */
+   /* Otherwise, look for possible crash-recovery files. */
+   /*----------------------------------------------------*/
 
    if (argc == 2 + (k != 0) || initargc == 2 + (k != 0)) {
       strcpy(_STR2, argv[(k == 1) ? 2 : 1]);
       startloadfile();
    }
+   else {
+      findcrashfiles();
+   }
 
    composelib(PAGELIB);	/* make sure we have a valid page list */
    composelib(LIBLIB);	/* and library directory */
@@ -2277,6 +2302,7 @@ int main(int argc, char *argv[])
    /*----------------------*/
 
    XtMainLoop();
+   return EXIT_SUCCESS;
 }
 
 /*-------------------------------------------------------------------------*/
