@@ -97,7 +97,7 @@
 
 /*----------------------------------------------------------------------*/
 
-#if !defined(_SYS_TYPES_H) || !defined(__USE_BSD)
+#ifndef HAVE_U_CHAR
 typedef unsigned char	u_char;
 typedef unsigned short	u_short;
 typedef unsigned int	u_int;
@@ -183,6 +183,17 @@ UIPARAM_MODE
 
 };
 
+/*----------------------------------------------------------------------*/
+/* File loading modes							*/
+/*----------------------------------------------------------------------*/
+
+#define LOAD_MODES 5
+
+#define IMPORT     1
+#define PSBKGROUND 2
+#define SCRIPT	   3
+#define RECOVER    4
+
 /*----------------------------------------------------------------------*/
 /* Text justification styles						*/
 /*----------------------------------------------------------------------*/
@@ -556,8 +567,9 @@ typedef struct _Portlist
 typedef struct _Calllist *CalllistPtr;
 typedef struct _Calllist
 {
-   objectptr callobj;
-   objinstptr callinst;
+   objectptr   callobj;
+   objinstptr  callinst;
+   int         devindex;	/* if non-zero, index of device in netlist */
    PortlistPtr ports;
    CalllistPtr next;
 } Calllist;
@@ -606,6 +618,7 @@ typedef struct _xcobject {
    u_char	schemtype;
    objectptr	symschem;	 /* schematic page support */
 
+   char		*devname;	 /* if non-null, name of device in netlist */
    NetlistPtr   netlist;
    PortlistPtr  portlist;
    CalllistPtr  calllist;
@@ -782,7 +795,7 @@ enum {
 
 #ifdef SCHEMA
    XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Info_Label, XCF_Connectivity,
-   XCF_Sim, XCF_SPICE, XCF_PCB,
+   XCF_Sim, XCF_SPICE, XCF_PCB, XCF_SPICEflat,
 #endif
    NUM_FUNCTIONS
 };
@@ -846,6 +859,7 @@ typedef struct {
    Boolean	mapped;		/* indicates if window is drawable */
 #ifdef SCHEMA
    Boolean	schemon;
+   Boolean	pinpointon;
 #endif
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
